from __future__ import annotations

from tcsoa.gen.Internal.Configurator._2015_10.ConfiguratorManagement import AvailableProductVariabilityInput, AvailableProductVariabilityOutput, GetVariantExpressionsResponse, ConvertVariantExpressionsResponse, ConvertVariantExpressionInput, ProductDefaultsInput, ValidateProductConfigInput, ValidateProductConfigurationResponse, GetRevisionRulesResponse, GetProductDefaultsResponse, SetVariantExpressionInput
from tcsoa.gen.BusinessObjects import BusinessObject, RevisionRule
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class ConfiguratorManagementService(TcService):

    @classmethod
    def convertVariantExpressions(cls, inputs: List[ConvertVariantExpressionInput]) -> ConvertVariantExpressionsResponse:
        """
        This operation converts the input application config configuration expressions  given in formulae and/or grid
        format into formulae, grid and displayable or user readable format.
        
        Use cases:
        1: Create an ApplicationConfigExpression structure providing only formula and not provide the grid expression 
        ConfigExpressionSet structure.
        2: Create an ApplicationConfigExpression structure providing the grid expression ConfigExpressionSet structure
        and not provide the formula.
        3: Create an ApplicationConfigExpression structure providing the formula as well as the grid expression
        ConfigExpressionSet structure.
        
        In all the three cases, the output ApplicationConfigExpression structure will be populated with formulae, grid
        expression ConfigExpressionSet structure and display formulae.
        
        
        The string "NS" is the family namespace. The family namespace defines the uniqueness of the family within
        itself. System ensures that the combination of the family namespace and the family id is unique in the
        database. For family "F" with namespace "NS" system will identify it as "[NS]F".
        
        The examples of display formulae are as follows:
        
        1)  Boolean expression
        
        [NS]BOOL - represents TRUE
        [NS]!BOOL - represents FALSE
        
        
        2)  Date expressions
        Date features are displayed in the standard Teamcenter locale-specific date format.
        3)  Range expressions
        
        [NS]A = a1 &amp; [NS]LENGTH <= 50
        [NS]A = a1 &amp; ([NS]LENGTH >= 1 &amp; [NS]LENGTH <= 10)
        [NS]A = a1 &amp; [NS]LENGTH >= 20
        [NS]A = a1 &amp; !([NS]LENGTH >= 20)
        
        
        4)  Multiple column expressions must be grouped within brackets
        
        ([NS]A = a1 &amp; [NS]B = b1) | ([NS]A = a2 &amp; [NS]C = c1)
        
        
        5)  Multiple selections within a single column
        
        ([NS]A = a1 | [NS]A = a2) &amp; [NS]B = b1
        
        
        6)  Multiple selections in a family 'C' that has Multi-select=Yes
        
        ([NS]A = a1 | [NS]A = a2) &amp; [NS]C = c1 &amp; [NS]C = c2
        
        
        7)  Optional family 'D' [Target]
        
        [NS]A = a1 &amp; [NS]D = NONE (same as [NS]D = '')
        [NS]A = a1 &amp; [NS]D = ANY (same as [NS]D != '')
        """
        return cls.execute_soa_method(
            method_name='convertVariantExpressions',
            library='Internal-Configurator',
            service_date='2015_10',
            service_name='ConfiguratorManagement',
            params={'inputs': inputs},
            response_cls=ConvertVariantExpressionsResponse,
        )

    @classmethod
    def getAvailableProductVariability(cls, input: AvailableProductVariabilityInput) -> AvailableProductVariabilityOutput:
        """
        This operation computes the available features within a set of requested families. The parameters
        criteriaExpression and familiesToTest are used to pass in the input expression and families of interest
        requesting the available features. The operation in response returns the available product variability for the
        requested families satisfying the input expression in criteriaExpression and the constraint(s) present in the
        system. If the input parameter criteriaExpression is empty, then the response will return the product
        variability for the requested families satisfying all the system constraints. If the requested family list is
        empty, the operation will not return any variability information.
        
        Use cases:
        Consider the following Variability data in a Configurator context say MyProductItem:
        
        Family&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;Features
        A
        &amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;A1
        &amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;A2
        &amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;A3
        
        B
        &amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;B1
        &amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;B2
        &amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;B3
        
        C
        &amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;C1
        &amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;C2
        &amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;C3
        
        Rules:
        &amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;A3 Includes B2:  This is an inclusion rule stating that if the
        configuration has the feature A3 (of the family A) then include the feature B2 as well (of the family B).
        B2 Excludes C2:  This is an exclusion rule state that if the configuration has the feature B2 (of the family B)
        then exclude/remove the feature C2 (of the family C)
        
        Use case 1:
        Operation getAvailableProductVariability is invoked with AvailableProductVariabilityInput populated as:
        CriteriaExpression = {}
        familiesToTest = {A, B, C}
        context containing reference to Item MyProductItem
        applyConstraints = 1(enable configurator constraint evaluation)
        
        Response would contain - A1, A2, A3, B1, B2, B3, C1, C2 &amp; C3
        
        Use case 2:
        Operation getAvailableProductVariability is invoked with AvailableProductVariabilityInput populated as:
        CriteriaExpression = {A=A3}
        FamiliesToTest = {B, C}
        context containing reference to Item MyProductItem
        applyConstraints = 1
        
        Response would - B2, C1 &amp; C3
        
        Use case 3:
        Operation getAvailableProductVariability is invoked with AvailableProductVariabilityInput populated as:
        criteriaExpression = {A=A1}
        FamiliesToTest = {B, C}
        context containing reference to Item MyProductItem
        applyConstraints = 1
        
        Response would - B1, B2, B3, C1, C2 &amp; C3
        
        
        Use case 4: 
        Initialize a variant criteria dialog for a new VariantRule in the Configurator context MyProductItem:
        The application uses operation getAvailableProductVariability with AvailableProductVariabilityInput populated
        as below:
        criteriaExpression = {}
        familiesToTest = {}
        context containing reference to Item MyProductItem
        ApplyConstraints = 0
        
        A service exception will be thrown by the  getAvailableProductVariability operation as familiesToTest is empty.
        """
        return cls.execute_soa_method(
            method_name='getAvailableProductVariability',
            library='Internal-Configurator',
            service_date='2015_10',
            service_name='ConfiguratorManagement',
            params={'input': input},
            response_cls=AvailableProductVariabilityOutput,
        )

    @classmethod
    def getProductDefaults(cls, inputs: List[ProductDefaultsInput]) -> GetProductDefaultsResponse:
        """
        This operation returns the product defaults for the input expression(s) by applying the default rule(s). The
        revision rule is used to determine the correct revision of configurator objects like rules, families, features,
        availability statements etc, that take part in the process. The parameter expressionsToUpdate specifies the
        input variant expressions for the variant configuration. The parameter families specifies the option families
        for which a default value assignment is requested. If specified, the server assigns default features for only
        these families in the response. If the list is empty, the server assigns default features for all the families.
        
        This operation also returns the list of mandatory families, which  do not have any value selection. The
        response returns this list of families in parameter requiredFamilies. The parameter criteriaStatus conveys the
        information about the status of the configuration.
        
        Use cases:
        Consider the following Variability data in a Configurator context :
        Family Features
        TYPE  CLASSIC, HATCHBACK, SEDAN
        ENGINE  L4, V6, V8
        COLOR RED, GREEN, BLUE
        ABS ABS
        Default Rules:
        Default Rule 1 - "If TYPE=CLASSIC and ENGINE=L4 then Apply COLOR=RED"
        Default Rule 2 - "If TYPE=CLASSIC and ENGINE=L4 then Apply ABS=TRUE"
        The getProductDefaults operation can be used in following ways:
        
        Example 1: 
        User provides the input expression to getProductDefaults as "TYPE=CLASSIC and ENGINE=L4" then the response will
        be "TYPE=CLASSIC and ENGINE=L4 and COLOR=RED and ABS=TRUE" in expressionsWithDefaults. The parameter
        requiredfamilies will be blank and the parameter criteiaStatus has value validAndComplete.
        
        Example 2:
        User provides input expression to getProductDefaults as TYPE=CLASSIC and ENGINE=L4, the input parameter
        families contains family COLOR. The response returns the variant expression as "TYPE=CLASSIC and ENGINE=L4 and
        COLOR=RED". ABS=TRUE is not expected in the response because family ABS didn't exist in the input parameter
        families. The parameter requiredfamilies will contain "ABS" and the parameter criteriaStatus has value
        validAndInComplete.
        
        Example 3:
        User provides input expression to getProductDefaults as TYPE=HATCHBACK and ENGINE=L4, the input parameter
        families contains family as COLOR. The response will have the variant expression as "TYPE=HATCHBACK and
        ENGINE=L4". The parameter requiredfamilies will contain the Family COLOR and the parameter criteriaStatus has
        value validAndInComplete. Consider the following Variability data in a Configurator context : Family Values
        Models  VXI, LXI TYPE CLASSIC, HATCHBACK, SEDAN  
        Availability Rules:  "TYPE = CLASSIC" is available to "Models = VXI"  "TYPE = SEDAN" is available to "Models =
        LXI"  Default Rules: Default Rule 1: "TYPE = SEDAN".
        
        Example 4:
        User provides input expression to getProductDefaults as "Models = VXI | Models = LXI".  The response will have
        the variant expression as "Models = VXI | Models = LXI &amp; TYPE = SEDAN". Here we have two branches in the
        input criteria expression.  The first branch is "Models = VXI" and the second branch is "Models = LXI". As
        SEDAN is not available to VXI and it is available to LXI, default rule gets applied to second branch only.
        The parameter requiredfamilies will contain blank and the parameter criteriaStatus has value validAndComplete.
        
        Exceptions:
        >No error conditions result at this time for this operation.
        """
        return cls.execute_soa_method(
            method_name='getProductDefaults',
            library='Internal-Configurator',
            service_date='2015_10',
            service_name='ConfiguratorManagement',
            params={'inputs': inputs},
            response_cls=GetProductDefaultsResponse,
        )

    @classmethod
    def getRevRulesForConfiguratorContext(cls) -> GetRevisionRulesResponse:
        """
        This operation provides the list of RevisionRule objects which can be used for configuration of the
        Configurator contexts in Product Configurator application. 
        
        The operation will return the RevisionRule instances. The supported rule entries in the Revision Rule mentioned
        below:
        - Has Released Status
        - Working 
        - Latest Created
        
        
        
        Use cases:
        When user queries the allocated features and families with the RevisionRule instance, the configured revisions
        of the families and features as per the input RevisionRule are returned.
        """
        return cls.execute_soa_method(
            method_name='getRevRulesForConfiguratorContext',
            library='Internal-Configurator',
            service_date='2015_10',
            service_name='ConfiguratorManagement',
            params={},
            response_cls=GetRevisionRulesResponse,
        )

    @classmethod
    def getVariantExpressions(cls, targetObjects: List[BusinessObject], revisionRule: RevisionRule) -> GetVariantExpressionsResponse:
        """
        This operation returns the variant expressions associated with the given set of target objects. The revision
        rule specified in 'revRule' is used to retrieve the correct revision of variant data referenced by variant
        expressions.  Setting revRule to nullTag is valid.  In this case  any unconfigured wso would be returned and
        caller should use it only for thread related properties like objectId, namespace, datatype, UOM.
        
        For example using a valid revRule, An inclusion rule has variant expression set as   "[Dict]familyA.feature1
        AND NOT [Dict]familyB.feature2" . The grid expression structure returned by this operation in the response will
        have values populated as follows
        
        Use cases:
        User can get variant expressions of an existing object. For example 
        
        Using any existing revision rule and using operation 'setVariantExpressions' set subject condition as "Color =
        Red". To retrieve this variant expression 'getVariantExpressions' can be used which returns variant expression
        formula and the client representable structure format.
        """
        return cls.execute_soa_method(
            method_name='getVariantExpressions',
            library='Internal-Configurator',
            service_date='2015_10',
            service_name='ConfiguratorManagement',
            params={'targetObjects': targetObjects, 'revisionRule': revisionRule},
            response_cls=GetVariantExpressionsResponse,
        )

    @classmethod
    def setVariantExpressions(cls, inputs: List[SetVariantExpressionInput], revisionRule: RevisionRule) -> ServiceData:
        """
        This operation sets a variant expression for the input business object. The variant expressions can be set for
        rules which includes include rule, exclude rule, default rule and availability rules, variant conditions and
        variant criteria. 
        
        On configurator rules multiple variant expressions or conditions can be set such as constraint and
        applicability condition. The variant expression is identified with the "'expressionType'" set on input.
        
        User can set transient or in memory variant expression on the target object.. The input flag saveExpressions
        can be used only for the objects having session recovery feature supported, for example VariantRule. If the
        parameter saveExpressions is set as TRUE then the variant expression object is saved and the persistent variant
        expression is returned. 
        
        If the saveExpressions is set to FALSE, then the operation will create a transient variant expression. If
        configuration need not be persisted and is for temporary use, the transient variant expression can be used. If
        a transient configuration expression is created, then user needs to save the target object explicitly or else
        the changes are lost when the user session ends. Transient changes or objects provide a convenient way to make
        temporary changes that automatically go out of scope when the session ends.
        
        Use cases:
        User can use this operation set variant expressions of an existing object. For example
        
        Create an instance of default rule (Cfg0DefaultRule) The operation 'setVariantExpressions' will set
        applicability condition as "Color = Red". User can provide variant expression for applicability condition in
        form of formula i.e. "[Dictionary]Color=Red" or populated grid expression service structure. 
        """
        return cls.execute_soa_method(
            method_name='setVariantExpressions',
            library='Internal-Configurator',
            service_date='2015_10',
            service_name='ConfiguratorManagement',
            params={'inputs': inputs, 'revisionRule': revisionRule},
            response_cls=ServiceData,
        )

    @classmethod
    def validateProductConfiguration(cls, inputs: List[ValidateProductConfigInput]) -> ValidateProductConfigurationResponse:
        """
        This operation validates the input expression(s) by applying the constraint rules such as inclusion rule,
        exclusion rule or availability rule for the input configurator perspective. The revision rule is used to
        determine the correct revision of configurator objects like rules, families, features, availability statements
        etc, that will take part in validation and configuration process. The parameter expressionsToValidate specifies
        the input variant expressions that needs to be validated. 
        
        This operation also returns the list of mandatory families, which don't have any feature selection. The
        response returns this list of families in parameter requiredFamilies. The parameter criteriaStatus conveys the
        information about the status of the configuration.
        
        Use cases:
        Consider the following Variability data and Rules in a Configurator context:
        Family  Features
        Model  Luxury, Economy
        TYPE  CLASSIC, HATCHBACK, SEDAN
        ENGINE  L4, V6, V8
        COLOR  RED, GREEN, BLUE
        # All the families are non-discretionary families.
        Rules:
        Exclude Rule 1 - If 'ENGINE=L4' then exclude 'TYPE=SEDAN else Error "TYPE cannot be SEDAN for L4 ENGINE".
        Include Rule 2 - If 'TYPE=CLASSIC' then include 'COLOR=RED' else Error "Classic comes with Red Color only".
        
        Use case 1:
        User provides the input expression to validateProductConfiuration as "TYPE=SEDAN and ENGINE=L4" to validate the
        expression. The output of the operation is a violation expected with message as "TYPE cannot be SEDAN for L4
        ENGINE" and updated expression as "TYPE=SEDAN and ENGINE=L4". The parameter requiredFamilies will be empty, as
        there are violations and parameter criteriaStatus has value inValid.
        
        Use Case 2:
        User creates an availability say 'RED' is available to model 'LUXURY'. User does not define any availability of
        the value 'BLUE' for the product model 'LUXURY'. System is configured to evaluate the availability constraints
        and the user provides an expression as "MODEL=LUXURY and COLOR=BLUE Then the output of the operation returns a
        violation with the message "The option value 'BLUE' is not available to the following product models: LUXURY".
        The parameter requiredFamilies will be empty, as there are violations and parameter criteriaStatus has value
        inValid.
        
        Use Case 3:
        User provides the input expression as "TYPE=CLASSIC" to validateProductConfiuration operation will validate the
        expression. Then the output of the operation returns the updated expression will "TYPE=CLASSIC and COLOR=RED"
        with no violation. The parameter requiredFamilies will contain family "ENGINE" and parameter criteriaStatus has
        value validAndInComplete. 
         
        Use Case 4:
        User provides the input expression as "TYPE=CLASSIC and COLOR=BLUE" The output of the operation returns a
        violation with message as "Classic comes with Red Color only". The parameter requiredFamilies will be empty, as
        there are violations and parameter criteriaStatus has value inValid.
        """
        return cls.execute_soa_method(
            method_name='validateProductConfiguration',
            library='Internal-Configurator',
            service_date='2015_10',
            service_name='ConfiguratorManagement',
            params={'inputs': inputs},
            response_cls=ValidateProductConfigurationResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Dataset, ItemRevision, ImanFile
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class InsertDatasetVersionArgs(TcBaseObj):
    """
    The InsertDatasetVersionArgs struct is used to pass multiple sets of data to be used in a single call.  These
    structures are passed in the collection of input arguments to the function insertDatasetVersion.
    
    :var dataset: The Dataset of you would like insert a version.
    :var versionNumber: The version number you would like to insert.
    """
    dataset: BusinessObject = None
    versionNumber: int = 0


@dataclass
class InsertDatasetVersionResponse(TcBaseObj):
    """
    The InsertDatasetVersionResponse struct is the output values for the call to the function insertDatasetVersion.
    
    :var datasetTags: A 2 element array where the first element represents the dataset version tag and the second
    element represents the zero (0) dataset tag.
    :var svcData: The service data needed to pass the above values back.
    """
    datasetTags: List[BusinessObject] = ()
    svcData: ServiceData = None


@dataclass
class InternalDataFiles(TcBaseObj):
    """
    This structure represents the key/file pairs that can be attached to the Dispatcher Request.
    
    :var key: The key of the key/file pair.
    :var file: file
    """
    key: str = ''
    file: ImanFile = None


@dataclass
class InternalKeyValueArguments(TcBaseObj):
    """
    This structure represents the key/value pairs that can be attached to the Dispatcher Request.
    
    :var key: The key of the key/value pair.
    :var value: The value of the key/value pair.
    """
    key: str = ''
    value: str = ''


@dataclass
class QueryDispatcherRequestsArgs(TcBaseObj):
    """
    The QueryDispatcherRequestsArgs is the possible data to be used in querying for DispatcherRequests.  Ant
    combination of the available data can be set.  If none of the values are set though, no items will be returned and
    the query will be skipped.
    
    :var providers: A list of providers.
    :var services: A list of services
    :var states: A list of states (i.e. INITIAL, TERMINAL)
    :var priorities: A list of priorities (i.e. 1,2,3)
    :var modifiedDate: A modified before date.  This is only a single date and not a list of them because that would
    create a range and not a before date.
    :var primaryObjects: The primary objects (i.e. Datasets)
    :var taskID: The task identifier of the request.  This is helpful for loading a specific request.
    :var type: The type of the request.
    :var unLoaded: If the service data should NOT be loaded with returning object.
    """
    providers: List[str] = ()
    services: List[str] = ()
    states: List[str] = ()
    priorities: List[int] = ()
    modifiedDate: str = ''
    primaryObjects: List[BusinessObject] = ()
    taskID: List[str] = ()
    type: List[str] = ()
    unLoaded: bool = False


@dataclass
class QueryDispatcherRequestsOutput(TcBaseObj):
    """
    The QueryDispatcherRequestsOutput struct is the output structure containing the requests that were returned from
    the corresponding input arguments to the query.
    
    :var queriedRequests: A list of the ETSDispatcherRequests returned from the query within the Teamcenter database.
    """
    queriedRequests: List[BusinessObject] = ()


@dataclass
class QueryDispatcherRequestsResponse(TcBaseObj):
    """
    The QueryDispatcherRequestsResponse struct is the output values for the call to the function
    queryDispatcherRequests.
    
    :var outputObjects: A list of the output data associated with this call.
    :var svcData: The service data needed to pass the above values back.
    """
    outputObjects: List[QueryDispatcherRequestsOutput] = ()
    svcData: ServiceData = None


@dataclass
class UpdateDispatcherRequestArgs(TcBaseObj):
    """
    The UpdateDispatcherRequestArgs is the possible data to be used in updating a Dispatcher Request object.
    
    :var requestToUpdate: The DispatcherRequest to update.  This value MUST be set and cannot be empty.
    :var currentState: The current state of the request.  This value is used as a safety check when setting the
    nextState of the request to make sure we have the current up to date request.
    :var nextState: The next state to assign to the request.
    :var type: type
    :var keyValueArgs: The key/value args to assign to the request.  This will override the current argument if it
    exist or add the new argument to the list.
    :var dataFiles: The data files to assign to the request.  This will override the current data file if it exist or
    add the new data file to the list.
    """
    requestToUpdate: BusinessObject = None
    currentState: str = ''
    nextState: str = ''
    type: str = ''
    keyValueArgs: List[InternalKeyValueArguments] = ()
    dataFiles: List[InternalDataFiles] = ()


@dataclass
class UpdateDispatcherRequestOutput(TcBaseObj):
    """
    The UpdateDispatcherRequestOutput struct is the output structure containing the request that was updated and a
    boolean value stating if the request was updated.
    
    :var requestUpdated: This is the request that was updated and should be the request that the client was previously
    referencing.
    :var wasRequestUpdated: This is a boolean representing whether or not the request was successfully updated.
    """
    requestUpdated: BusinessObject = None
    wasRequestUpdated: bool = False


@dataclass
class UpdateDispatcherRequestResponse(TcBaseObj):
    """
    The UpdateDispatcherRequestResponse struct is the output values for the call to the function
    updateDispatcherRequest.
    
    :var outputObjects: A list of the output data associated with this call.
    :var svcData: The service data needed to pass the above values back.
    """
    outputObjects: List[UpdateDispatcherRequestOutput] = ()
    svcData: ServiceData = None


@dataclass
class CreateDatasetOfVersionArgs(TcBaseObj):
    """
    The CreateDatasetOfVersionArgs is used to carry the data needed to create a dataset of a particular version.  This
    is similar to the core create dataset method with the exception that this method takes a specific version and
    creates the dataset at that version even if it is the first instance of that dataset.
    
    :var itemRevision: The ItemRevision to relate the new Dataset to once created.
    :var datasetName: The name of the Dataset to create.
    :var typeName: The type of the Dataset to create.
    :var tool: The Tool to associate with the new Dataset.
    :var version: The version to assign to the new Dataset.
    """
    itemRevision: ItemRevision = None
    datasetName: str = ''
    typeName: str = ''
    tool: BusinessObject = None
    version: int = 0


@dataclass
class CreateDatasetOfVersionResponse(TcBaseObj):
    """
    The CreateDatasetOfVersionResponse struct is the output values for the call to the function createDatasetOfVersion.
    
    :var createdDatasets: A list of the Datasets created corresponding to the list of input data.
    :var svcData: The service data needed to pass the above values back.
    """
    createdDatasets: List[Dataset] = ()
    svcData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Internal.GlobalMultiSite._2010_02.LowlevelOwnershipTransfer import ObjectsForOwnershipTransferResponse
from tcsoa.utils import TcService


class LowlevelOwnershipTransferService(TcService):

    @classmethod
    def getObjectsForOwnershipTransfer(cls, tcGSMessageId: str, changeOwnershipToSite: int, dryrun: bool, startDate: str, endDate: str) -> ObjectsForOwnershipTransferResponse:
        """
        This is an internal operation. It generates a report of source objects which were replicated to target site
        (specified in changeOwnershipToSite) between start date and end date using site consolidation tool. Date range
        is valid only for dry run mode. The report has the following format
        #Accountability Data
        # puids for ownership transfer (low level)
        # Syslog-FileName    :  C:\Temp\tcxml_xfer_ownership26585f2e.syslog
        #$DryRun: Yes
        #$SourceSiteID: 100001
        #$TargetSiteID: 100002
        #PUID         IslandAnchorUIDs
        QLK53SsdAABaaA ArD53SsdAABaaA
        QLG53SsdAABaaA ArD53SsdAABaaA
        ArN53SsdAABaaA ArD53SsdAABaaA
        
        
        Use cases:
        It is invoked by BPEL process of Global services when user runs sitcons_xfer_owner_mgr utility of site
        consolidation.
        """
        return cls.execute_soa_method(
            method_name='getObjectsForOwnershipTransfer',
            library='Internal-GlobalMultiSite',
            service_date='2010_09',
            service_name='LowlevelOwnershipTransfer',
            params={'tcGSMessageId': tcGSMessageId, 'changeOwnershipToSite': changeOwnershipToSite, 'dryrun': dryrun, 'startDate': startDate, 'endDate': endDate},
            response_cls=ObjectsForOwnershipTransferResponse,
        )



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from typing import Dict


@dataclass
class GetCurrentCountryPageInfoResponse(TcBaseObj):
    """
    The information used to populate the current country selection page.
    
    :var displayCountries: Map of key/value pairs (string, string) representing the selectable countries the user may
    choose when filling out the current country selection page.
    - Key is the ISO 3166-1 alpha-2 two letter country codes.
    - Value is the full country name.
    
    
    Examples: US &ndash; United States of America, GB &ndash; United Kingdom, and IN &ndash; India.
    :var extraInfoOut: Map of key/value pairs (string, string).
    - Key: initialCountry. The initial value displayed in the current country pick list. The format is ISO 3166-1
    alpha-2 two letter country. This is optional. The value may be the empty string.  
    - Key: confidentialityStatement. The agreement that the user must read when filling out the current country page.
    
    
    :var serviceData: ServiceData.
    """
    displayCountries: ExtraMappedInfo = None
    extraInfoOut: ExtraMappedInfo = None
    serviceData: ServiceData = None


"""
Map of information used to populate the current country page.
"""
ExtraMappedInfo = Dict[str, str]



from __future__ import annotations

from typing import List
from tcsoa.gen.Internal.ActiveWorkspaceVis._2015_03.MassiveModelVisualization import ProductAndConfigInfo
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class ProductStructureIdInfo2(TcBaseObj):
    """
    This structure contains the id of the product structure and the configuration information that need to be applied
    to the product structure.
    
    :var clientId: A clientId passed by the client in order to map the input to the corresponding output in the
    response.
    :var productStrutureIdentifier: The identifier to identify the product structure information. This identifier
    corresponds to the product id in the product structure(mmp file).
    :var productAndConfig: The configuration that need to be applied on the product structure.
    :var clientRevRuleInfo: The client revision rule information to be used for solver library inside VDS.
    :var partitionRevRuleInfo: The Partition revision rule information to be used for solver library inside VDS.
    """
    clientId: str = ''
    productStrutureIdentifier: str = ''
    productAndConfig: ProductAndConfigInfo = None
    clientRevRuleInfo: RevRuleInfo = None
    partitionRevRuleInfo: RevRuleInfo = None


@dataclass
class RevRuleInfo(TcBaseObj):
    """
    This structure contains names of revision rule entry names, effectivity expression, variant expression and revision
    rule date.
    
    :var revRuleEntryNames: A list of revision rule entry names.
    :var effectivityExpr: Effectivity expression string.
    :var variantExpr: Variant expression string.
    :var revRuleDate: Revision rule Date.
    """
    revRuleEntryNames: List[str] = ()
    effectivityExpr: str = ''
    variantExpr: str = ''
    revRuleDate: datetime = None


@dataclass
class GetProductStructureIdResponse2(TcBaseObj):
    """
    This structure contains the information about the product structure information identifier which can be used to
    determine the product structure information from the catalogue of product structure information maintained at the
    client. The configuration that need to be applied on that product structure information is also returned via this
    response.
    
    :var psIdInfos: A list of product and configuration for which the product structure information are indexed.
    :var serviceData: ServiceData by which the partial errors are communicated to the client.
    """
    psIdInfos: List[ProductStructureIdInfo2] = ()
    serviceData: ServiceData = None



import dataclasses
import json
import logging
import re
import sys
from enum import Enum
from typing import List, Dict, Iterable

import requests

from datetime import datetime

from tcsoa.config import TcSoaConfig
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBoClass


class RestBackend:
    def __init__(self, host: str):
        """
        Initializes the REST Backend. Should be useable for all Teamcenter Backends which have AWC installed.

        @param host: The address to your web server - for example: http://webserver:7001/tc
        """
        self.host = host
        self.base_url = f'{self.host}/RestServices'
        self.policy = {
            "useRefCount": False,
            "types": list()
        }
        self.state = {
            "stateless": True,
            "unloadObjects": True,
            "enableServerStateHeaders": True,
            "formatProperties": True,
            "clientID": "TcSoaClient"
        }
        self.session = requests.session()
        self.session.trust_env = False
        self.session.proxies = {
            "http": None,
            "https": None,
        }

    def set_object_load_policy(self, bo_name, properties: List[str]):
        types_list = self.policy['types']
        matching_type = list(t for t in types_list if t['name'] == bo_name)
        if matching_type:
            object_policy = matching_type[0]
        else:
            object_policy = {
                'name': bo_name
            }
            types_list.append(object_policy)
        object_policy['properties'] = [{'name': p} for p in properties]

    def input_obj_to_json(self, input_obj):
        if input_obj is None:
            return ''
        if type(input_obj) in (str, int, float, bool):
            return input_obj
        if isinstance(input_obj, TcBoClass):
            return {k: self.input_obj_to_json(v) for k, v in input_obj.__dict__.items() if k != 'props'}
        if dataclasses.is_dataclass(input_obj):
            return {k[:-1] if k.endswith('_') else k: self.input_obj_to_json(v) for k, v in input_obj.__dict__.items()}
        if isinstance(input_obj, datetime):
            return input_obj.strftime(TcSoaConfig.date_format)
        if isinstance(input_obj, Enum):
            return input_obj.value
        if isinstance(input_obj, dict):
            return {k: self.input_obj_to_json(v) for k, v in input_obj.items()}
        if type(input_obj) in (list, set, tuple) or isinstance(input_obj, Iterable):
            return [self.input_obj_to_json(o) for o in input_obj]
        raise NotImplementedError()

    def qname2type(self, qname):
        # example: http://teamcenter.com/Schemas/Core/2011-06/Session.LoginResponse
        qname_match = re.search(r'Schemas\/[a-zA-Z0-9]*\/[0-9]{4}-[0-9]{2}\/[a-zA-Z0-9]*$', qname)
        assert qname_match
        _, library, date, service, typename = qname_match.group().replace('.', '/').split('/')
        # https://stackoverflow.com/questions/8790003/dynamically-import-a-method-in-a-file-from-a-string
        # todo: add cache for qname types

    def response_to_output_obj(self, response_cls, response):
        try:
            response_data: Dict[str: any] = response.json()
            qname = response_data.pop('.QName')
            # todo: some kind of check...
            # todo: also - what to do when response_cls in None??
            if response_cls:
                response_data = self.dataclass_from_dict(response_cls, response_data)
            return response_data
        except ValueError:
            # note to self: when XML is returned, you dont have a valid cookie
            return response.content  # todo: handle this better

    @staticmethod
    def eval_str_type(cls, field):
        """ Black Magic - Evaluates the type annotation in context of the class """
        return eval(field, globals(), sys.modules[cls.__module__].__dict__)

    @staticmethod
    def dataclass_from_dict(cls, from_dict):
        try:
            fieldtypes = cls.__annotations__
        except AttributeError:
            if dataclasses.is_dataclass(cls):
                fieldtypes = dict()     # __annotations__ is only set to data classes with attributes
            elif isinstance(from_dict, (tuple, list)):
                return [RestBackend.dataclass_from_dict(cls.__args__[0], f) for f in from_dict]
            elif isinstance(from_dict, dict):
                return {
                    RestBackend.dataclass_from_dict(cls.__args__[0], k):
                        RestBackend.dataclass_from_dict(cls.__args__[1], v)
                    for k, v in from_dict.items()
                }
            else:
                # logging.warning(f'Warning! Expected attribute class {str(cls)} but received {from_dict} !')
                return from_dict

        instance = cls()
        for f in from_dict:
            dest_f = f
            if f == 'ServiceData' and 'serviceData' in fieldtypes:
                dest_f = 'serviceData'

            if dest_f in fieldtypes:
                field = fieldtypes[dest_f]
            else:
                logging.debug(f'Warning! unregistered attribute "{f}" is tried to be set on class "{cls.__name__}"!')
                setattr(instance, dest_f, from_dict[f])
                continue

            if isinstance(field, str):  # black magic incoming
                field = RestBackend.eval_str_type(cls, field)
            setattr(instance, dest_f, RestBackend.dataclass_from_dict(field, from_dict[f]))
        if TcSoaConfig.global_obj_cache_enabled:
            if isinstance(instance, ServiceData):
                TcSoaConfig.internal_handle_sd(instance)
        return instance

    def execute(self, service_id, method_name, input_obj, response_cls):
        json_body = dict(
            header=dict(
                state=self.state,
                policy=self.policy,
            ),
            body=self.input_obj_to_json(input_obj)
        )
        url = f'{self.base_url}/{service_id}/{method_name}'
        data = json.dumps(json_body, ensure_ascii=False).encode('utf8')
        response = self.session.post(url, data=data, headers={
            'Operation-Name': method_name,
            'App-Xml': 'application/json',
            'Content-Type': 'application/json',
        })
        response.encoding = "utf-8"
        response_obj = self.response_to_output_obj(response_cls, response)
        return response_obj



from __future__ import annotations

from tcsoa.gen.Core._2018_06.DataManagement import CreateIn3
from tcsoa.gen.Core._2008_06.DataManagement import CreateResponse
from tcsoa.gen.Core._2018_06.LogicalObject import GetLogicalObjectResponse2, GetLogicalObjectInput2
from typing import List
from tcsoa.utils import TcService


class DataManagementService(TcService):

    @classmethod
    def createObjectsInBulkAndRelate(cls, createInputs: List[CreateIn3]) -> CreateResponse:
        """
        (1)    This is a generic operation for creation of business objects in bulk (i.e. in a set&ndash;based manner).
        All business objects will be created or none, a failure for a single object will result in no business objects
        being created.
        This operation will also create any secondary objects compounded with the main object being created. For
        example, please refer to the Create Item (a compound object) use case.
        (2)    The other task this operation will perform is to relate the created business object to the input target
        object.
        
        Use cases:
        This operation is typically preceded by a call to
        Teamcenter::Soa::Core::_2008_06::PropDescriptor::getCreateDesc or to the Client Meta Model layer to retrieve
        Create Descriptor(s) for business objects to be created.
        The CreateInput2 object is used to hold the Create Descriptor info.
        (1)      Create Item (a compound object)
        Item is a compound object. Item creation in turn creates Item Master and ItemRevision. ItemRevision creation in
        turn creates ItemRevision Master.
        This means the CreateInput2 object for the Item will in turn hold two CreateInput2 objects (in the
        CreateInputMap2 one each for "IMAN_master_form" and "revision" respectively), for Item Master and ItemRevision.
        Also, the CreateInput2 object for the ItemRevision will in turn hold CreateInput2 object (in the
        CreateInputMap2 for "IMAN_master_form_rev") for ItemRevision Master.
        (2)      Relate the created object to the input target object
        A created object is linked to the input target object (targetObject) for the input property (pasteProp). In
        case, the targetObject is empty, it would default to the Newstuff Folder or Home Folder for the session User
        depending on the value for the preference, WsoInsertNoSelectionsPref. In case, the pasteProp is empty,
        following sequence is used to come up with a preferred property name,
        (a)     If created object is of type, WolfObject and the targetObject is a Folder, pasteProp will be set to
        "TC_external_object_link".
        (b)     The format, <targetObjectTypeName>_<createdObjectTypeName>_default_relation will be used to search for
        a preference. The whole parent Type hierarchy for both, the targetObject and the createdObject will be
        searched. In case, such a preference is present, pasteProp will be set to that preference value.
        (c)     The format, <targetObjectTypeName>_default_relation will be used to search for a preference. The whole
        parent Type hierarchy for the targetObject will be searched. In case, such a preference is present, pasteProp
        will be set to that preference value.
        (d)     In case, the targetObject is a Folder, pasteProp will be set to "contents".
        (e)     Else, an error will be reported in the response.
        
        Exceptions:
        >214200 &ndash; Unable to create business objects.
        """
        return cls.execute_soa_method(
            method_name='createObjectsInBulkAndRelate',
            library='Core',
            service_date='2018_06',
            service_name='DataManagement',
            params={'createInputs': createInputs},
            response_cls=CreateResponse,
        )


class LogicalObjectService(TcService):

    @classmethod
    def getLogicalObjects2(cls, loInputs: List[GetLogicalObjectInput2]) -> GetLogicalObjectResponse2:
        """
        This operation returns the logical object instances for the input list of root object instances and logical
        object type names. This operation can also return classification objects (A classification object is also
        called an ICO), if the root obejct or a member object is configured on the logical object type for retieving
        classification data. For such use cases it also returns ICO property data in an ICO specific property structure.
        
        This operation will have a one-to-one mapping between the input list   and output list.  The order in the input
        list is also matched to the order in the output list. In an error scenario, an empty entry in the output list 
        will be returned.
        
        Use cases:
        This operation is invoked to retrieve logical object instances by the client by passing a list of root object
        instances and logical object type names.
        
        For example, consider the below use case data for the use case(s) specified:
        Use Case Data:
        
        1. Logical object definition 1
        Name:  "Logical Object 1"
        Root business object: Item
        
        2. Logical object definition 2
        Name: "Logical Object 2"
        Root business object : ItemRevision 
        
        3. Logical object definition 3
        Name : "Logical Object 3"
        Root business object: Item
        
        4. Logical object definition 4
        Name : "Logical Object 4"
        Root business object : ItemRevision
        Retrieve classification data : True
        
        Use Case 1:
        
        GetLogicalObjectInput[0] =
        { 
           {   rootObject = Item,
                clientID = "UniqueID-1"
           } 
           loTypeName = "Logical Object 1"
        }
        
        then GetLogicalObjectResponse2 structure would contain loOutputs list of size = 1
        and element = 1 contains 1 instance(s) of "Logical Object 1"
        
        Use Case 2:
        
        GetLogicalObjectInput[0] = 
        { 
           {   rootObjects = Item,
                  clientID = "UniqueID-2"
             } 
             loTypeName = "Logical Object 2"
        }
        
        then GetLogicalObjectResponse2 structure would contain loOutputs list of size = 1
        and element = 1 contains 0 instance(s)
        
        Error 39040 - Is stored in the partial error list of service data with "Logical Object type name +  root object
        client id" as the key which helps the caller identify for which logical object type and root combination, the
        logical object instance is not created.
        Since Root business object for "Logical Object 2" is ItemRevision, system can&rsquo;t create a Logical Object
        instance with Item instance as root object input.
        
        Use Case 3:
        GetLogicalObjectInput[0] =
        { 
             {  rootObjects = Item
                  clientID = "UniqueID-3"
             } 
             {  rootObjects = Item
                  clientID = "UniqueID-4"
             } 
             loTypeName = "Logical Object 1"
        }
        
        then GetLogicalObjectResponse2 structure would contain loOutputs list of size = 1
        and element = 1 contains 2 instance(s) of "Logical Object 1" and "Logical Object 1"
        
        Use Case 4:
        
        GetLogicalObjectInput[0] = 
        { 
             {  rootObjects = Item
                  clientID = "UniqueID-4"
             } 
             {  rootObjects = ItemRevision
                  clientID = "UniqueID-5"
             }
             loTypeName = "Logical Object 1"
        }
        
        then GetLogicalObjectResponse2 structure would contain loOutputs list of size = 1
        and element = 1 contains 1 instance(s) of "Logical Object 1"
        
        Error 39040 - Is stored in the partial error list of service data with "Logical Object type name +  root object
        client id" as the key which helps the caller identify for which logical object type and root combination, the
        logical object instance is not created.
        Since Root business object for "Logical Object 1" is Item, system can&rsquo;t create a Logical Object instance
        with ItemRevision instance as root object input.
        
        Use Case 5:
        
        GetLogicalObjectInput[0] = 
        {
           {  rootObjects = Item
                clientID = "UniqueID-6"
           }
           loTypeName = "Logical Object 1"
        }
        GetLogicalObjectInput[1] = 
        { 
           {  rootObjects = ItemRevision
                clientID = "UniqueID-7"
           }
           loTypeName = "Logical Object 2"
        }
        
        then GetLogicalObjectResponse2 structure would contain loOutputs list of size = 2
        and element = 1 returns 1 instance(s) of "Logical Object 1"
        and element = 2 returns 1 instance(s) of "Logical Object 2"
        
        Use Case 6:
        
        GetLogicalObjectInput[0] =
        { 
           {  rootObjects = Item
                clientID = "UniqueID-8"
           }
           loTypeName = "Logical Object 1"
        }
        GetLogicalObjectInput[1] =
        { 
           {  rootObjects = Item
                clientID = "UniqueID-9"
           }
           loTypeName = "Logical Object 2"
        }
        GetLogicalObjectInput[2] =
        { 
           {  rootObjects = Item
                clientID = "UniqueID-10"
           }
           loTypeName = "Logical Object 3"
        }
        
        then GetLogicalObjectResponse2 structure would contain loOutputs list of size = 3
        and element = 1 returns 1 instance(s) of "Logical Object 1"
        and element = 2 contains 0 instance(s)
        and element = 3 contains 1 instance(s) of "Logical Object 3"
        
        Error 39040 - Is stored in the partial error list of service data with "Logical Object type name +  root object
        client id" as the key which helps the caller identify for which logical object type and root combination, the
        logical object instance is not created.
        Since Root business object for "Logical Object 2" is ItemRevision, system can&rsquo;t create a Logical Object
        instance with Item instance as root object input.
        
        Use Case 7:
        GetLogicalObjectInput[0] = 
        { 
           {  rootObjects = ItemRevision
                clientID = "UniqueID-11"
           }
           loTypeName = "Logical Object 4"
        }
        
        then GetLogicalObjectResponse2 structure would contain 
        loOutputs list of size = 1 
        and element = 1 returns 1 instance(s) of "Logical Object 4"
        GetLogicalObjectOutput2 structure would contain LogicalObjectOutput2 and then LogicalObject will provide
        details for the ICO objects assocaiated with ItemRevison and ClassificationAttributeDescMap2 map would provide
        details for the classification objects assocaiated with ItemRevison.
        """
        return cls.execute_soa_method(
            method_name='getLogicalObjects2',
            library='Core',
            service_date='2018_06',
            service_name='LogicalObject',
            params={'loInputs': loInputs},
            response_cls=GetLogicalObjectResponse2,
        )



from __future__ import annotations

from tcsoa.gen.Internal.ActiveWorkspaceBom._2015_03.OccurrenceManagement import AttachObjectsResp, AttachObjectsInputData
from typing import List
from tcsoa.utils import TcService


class OccurrenceManagementService(TcService):

    @classmethod
    def attachObjects(cls, input: List[AttachObjectsInputData]) -> AttachObjectsResp:
        """
        This operation attaches the secondary object with the primary with the given relation. If the context is
        specified then secondary object is associated in-context.
        
        Use cases:
        User wants to attach an object with occurrence present in the product.
        """
        return cls.execute_soa_method(
            method_name='attachObjects',
            library='Internal-ActiveWorkspaceBom',
            service_date='2015_03',
            service_name='OccurrenceManagement',
            params={'input': input},
            response_cls=AttachObjectsResp,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ItemRevision, RevisionRule, BOMLine
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ItemElementLineInfo(TcBaseObj):
    """
    This contains Item element Input structure for addOrUpdateChildrenToParentLine operation.
    
    :var clientId: Identifier that helps the client to track the object created. This is an optional parameter.
    :var itemElement: Refers to the Item element object.
    :var occType: The occurrence type used for the child BOMLine creation objects.
    :var itemElementline: Refers to BOMLine object which represents itemElement for modification of properties (Used in
    case of update).
    :var itemElementLineProperties: Refers to the 'BomLineProperties' struct.
    """
    clientId: str = ''
    itemElement: BusinessObject = None
    occType: str = ''
    itemElementline: BOMLine = None
    itemElementLineProperties: BomLineProperties = None


@dataclass
class ItemLineInfo(TcBaseObj):
    """
    Refers to Item input structure for addOrUpdateChildrenToParentLine operation.
    
    :var clientId: Identifier that helps the client to track the object created. This is an optional parameter.
    :var item: Refers to Item object.(used in case of precise structure)
    :var itemRev: Refers to the ItemRevision object (used in case of imprecise structure)
    :var occType: Refers to occurrence type used for the child BOMLine (occurrence) creation.
    :var bomline: Refers to BOMLine for modification of its properties (Used in case of update when clientId is empty).
    :var itemLineProperties: Refers 'BomLineProperties'  struct which represents to property name/value pairs for
    additional properties.
    """
    clientId: str = ''
    item: BusinessObject = None
    itemRev: BusinessObject = None
    occType: str = ''
    bomline: BOMLine = None
    itemLineProperties: BomLineProperties = None


@dataclass
class BOMLinesOutput(TcBaseObj):
    """
    This represents output structure for addOrUpdateChildrenToParentLine operation.
    
    :var clientId: Identifier that helps the client to track the object created.
    :var bomline: The refers to BOMLine object.
    """
    clientId: str = ''
    bomline: BOMLine = None


@dataclass
class RemoveChildrenFromParentLineResponse(TcBaseObj):
    """
    Return structure for removeChildrenFromParentLine operation
    
    :var serviceData: The ServiceData structure is used to return the updated parent BOMLine business objects whose
    children have been removed and can contain partial errors if the operations fails to create bom window. It also
    holds services exceptions.
    """
    serviceData: ServiceData = None


@dataclass
class BaselineInput(TcBaseObj):
    """
    Input structure that holds information to create a new Baseline ItemRevision
    
    :var dryrun: To be set as 'true' if dryrun is to be performed, false if not. This is an optional element and the
    default value is false. Dry run option helps users to know of any possible errors without performing the actual
    baseline action. Choosing this option generates a report and can be accessed from the 'BaselineResponse'.
    :var clientID:  Identifier that helps the client to track the object(s) created
    :var itemRev: Input ItemRevision object, that is to be baselined
    :var viewType: View type name. To be provided if input ItemRevision has BOMViewRevision
    :var revRule: RevisionRule object. To be provided if input ItemRevision has BOMViewRevision.
    :var precise: Creates a precise baseline if set to true. If set to false, creates an imprecise baseline. Default
    value is false.
    :var releaseProcess: Name of the workflow process template to be used for baselining.
    :var description: Description for baseline ItemRevision, optional
    :var baselineJobName: Name to identify the job initiated during baseline. Operation will fail if a job name is not
    provided. In general job name is a combination of ItemId, Revision Id and ItemRevision Name property values.
    :var baselineJobDescription: Description for baseline job, optional
    """
    dryrun: bool = False
    clientID: str = ''
    itemRev: ItemRevision = None
    viewType: str = ''
    revRule: RevisionRule = None
    precise: bool = False
    releaseProcess: str = ''
    description: str = ''
    baselineJobName: str = ''
    baselineJobDescription: str = ''


@dataclass
class BaselineOutput(TcBaseObj):
    """
    Refers to the output structure for baseline create operation.
    
    :var dryrun: BaselineOutput structure contains following elements
    dryrun - Dry run indicates that the operation will not create a baseline but it will only do the required
    validation. Boolean variable indicating if dry option was used.
    
    :var clientID: Client Identifier
    :var baselineItemRev: Created baseline ItemRevision object.
    :var dryrunLogTicket: FMS ticket for dryrun log. Contains path to dry run report if dryrun flag is set to true.
    """
    dryrun: bool = False
    clientID: str = ''
    baselineItemRev: ItemRevision = None
    dryrunLogTicket: str = ''


@dataclass
class BaselineResponse(TcBaseObj):
    """
    Output structure containing list of 'BaselineOutput' structures and 'ServiceData' with list of errors encountered
    during the Operation
    
    :var output: List of 'BaselineOutput' structures
    :var serviceData: structure containing error codes and messages
    """
    output: List[BaselineOutput] = ()
    serviceData: ServiceData = None


@dataclass
class AddOrUpdateChildrenToParentLineInfo(TcBaseObj):
    """
    Input structure for addOrUpdateChildrenToParentLine operation
    
    :var parentLine: Parent BOMLine business object under which item or item element occurrences are added or modified.
    :var viewType: View Type string used for creating BOMView for parent BOMLine if it does not exist (NULL implies use
    default view type).
    :var items: Array of ItemLineInfo input structure.
    :var itemElements: Array of ItemElementLineInfo input structure
    """
    parentLine: BOMLine = None
    viewType: str = ''
    items: List[ItemLineInfo] = ()
    itemElements: List[ItemElementLineInfo] = ()


@dataclass
class AddOrUpdateChildrenToParentLineResponse(TcBaseObj):
    """
    Return structure for addOrUpdateChildrenToParentLine operation
    
    :var itemLines: Array of Output itemLines
    :var itemelementLines: Array of Output itemElementLines
    :var serviceData: This is a common data strucuture used to return sets of
    Teamcenter Data Model object from a service request. This also
    holds services exceptions.
    """
    itemLines: List[BOMLinesOutput] = ()
    itemelementLines: List[BOMLinesOutput] = ()
    serviceData: ServiceData = None


"""
Refers to a map of BOMLine object property as key and BOMLine object property value as value pair.
"""
BomLineProperties = Dict[str, str]



from tcsoa.gen.Reports._2007_06.services import CubeReportsService as imp0
from tcsoa.gen.Reports._2007_01.services import CrfReportsService as imp1
from tcsoa.gen.Reports._2015_03.services import CrfReportsService as imp2
from tcsoa.gen.Reports._2008_12.services import CrfReportsService as imp3
from tcsoa.gen.Reports._2010_04.services import CrfReportsService as imp4
from tcsoa.gen.Reports._2015_10.services import CrfReportsService as imp5
from tcsoa.gen.Reports._2008_06.services import CrfReportsService as imp6
from tcsoa.utils import TcService


class CubeReportsService(TcService):
    constructReportURL = imp0.constructReportURL


class CrfReportsService(TcService):
    createReportDefinition = imp1.createReportDefinition
    generatePrintReports = imp2.generatePrintReports
    generatePrintReportsAsync = imp2.generatePrintReportsAsync
    generateReport = imp1.generateReport
    generateReport2 = imp2.generateReport
    generateReportAsync = imp2.generateReportAsync
    generateReportDefintionIds = imp1.generateReportDefintionIds
    generateReports = imp3.generateReports
    generateReports2 = imp4.generateReports
    getOfficeStylesheets = imp5.getOfficeStylesheets
    getPrintTemplates = imp2.getPrintTemplates
    getReportDefinitions = imp1.getReportDefinitions
    getReportDefinitions2 = imp6.getReportDefinitions



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Dataset, ImanRelation
from typing import List
from tcsoa.gen.Core._2007_06.DataManagement import RelationAndTypesFilter
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExpandGRMRelationsData2(TcBaseObj):
    """
    The data returned from 'expandGRMRelationsPrimary' and 'expandGRMRelationsSecondary' operations.
    
    :var relationshipObjects: The list of relation objects for the relationships between the input object and side
    objects.
    :var relationName: The input Generic Relationship Manager (GRM) relation type name.
    """
    relationshipObjects: List[ExpandGRMRelationship] = ()
    relationName: str = ''


@dataclass
class ExpandGRMRelationsOutput2(TcBaseObj):
    """
    The output from 'expandGRMRelationsPrimary' and 'expandGRMRelationsSecondary' operations.
    
    :var inputObject: The object that was input to be expanded.
    :var relationshipData: The list of data for relationships between the input object and found side objects.
    """
    inputObject: BusinessObject = None
    relationshipData: List[ExpandGRMRelationsData2] = ()


@dataclass
class ExpandGRMRelationsPref2(TcBaseObj):
    """
    Preference structure for 'expandGRMRelationsPrimary' and 'expandGRMRelationsSecondary'.
    
    :var expItemRev: Flag that if true signifies that any item revisions that are in the return data should be expanded.
    :var returnRelations: Flag that if true signifies that the relation objects should be returned.
    :var info: The list of relation type and primary or secondary object type filters, depending on whether
    'expandGRMRelationsForPrimary' or 'expandGRMRelationsForSecondary' is called.  This input must be specified or
    error 214160 will be returned.  For 'expandGRMRelationsForPrimary', if all secondary objects should be returned,
    'RelationAndTypesFilter' input parameters should be empty.  For 'expandGRMRelationsForSecondary', if all primary
    objects should be returned, 'RelationAndTypesFilter' input parameters should be empty.
    """
    expItemRev: bool = False
    returnRelations: bool = False
    info: List[RelationAndTypesFilter] = ()


@dataclass
class ExpandGRMRelationsResponse2(TcBaseObj):
    """
    The response from 'expandGRMRelationsPrimary' and 'expandGRMRelationsSecondary' operations.
    
    :var output: The list of input objects and the found related side objects.
    :var serviceData: The 'ServiceData'.  This operation will populate the 'ServiceData' with the input object, any
    found side object and any found relation object.  All objects are added as plain objects.
    """
    output: List[ExpandGRMRelationsOutput2] = ()
    serviceData: ServiceData = None


@dataclass
class ExpandGRMRelationship(TcBaseObj):
    """
    The relation information returned from 'expandGRMRelationsPrimary' and 'expandGRMRelationsSecondary' operations.
    
    :var otherSideObject: The found side object.
    :var relation: The found relation object for the side object to primary object relationship.
    """
    otherSideObject: BusinessObject = None
    relation: ImanRelation = None


@dataclass
class NamedReferenceInfo(TcBaseObj):
    """
    This structure contains information for NamedReference to be removed.
    
    :var clientId: A unique string supplied by the caller.  This ID is used to identify return data elements and
    partial errors associated with this input structure.  If 'clientId' is not provided then it can be difficult to
    align the input with any returned errors.
    :var type: Remove all 'NamedReference's of this reference name ( required )
    :var targetObject: Target object of a specific 'NamedReference' to remove ( optional, must match type above ) ) If
    specified then only this specific 'NamedReference' will be removed; other 'NamedReferences' of the same type will
    not be effected.
    :var deleteTarget: Flag to indicate if 'targetObject' is to be deleted
    """
    clientId: str = ''
    type: str = ''
    targetObject: BusinessObject = None
    deleteTarget: bool = False


@dataclass
class RemoveNamedReferenceFromDatasetInfo(TcBaseObj):
    """
    Input structure for the 'removeNamedReferenceFromDataset' operation
    
    :var clientId: A unique string supplied by the caller.  This ID is used to identify return data elements and
    partial errors associated with this input structure.  If 'clientId' is not provided then it can be difficult to
    align the input with any returned errors.
    :var dataset: The dataset object from which to remove the specified named references.
    :var nrInfo: A list of named reference information.
    """
    clientId: str = ''
    dataset: Dataset = None
    nrInfo: List[NamedReferenceInfo] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Manufacturing._2009_10.DataManagement import CreateInput
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ApplyConfigInput(TcBaseObj):
    """
    Structure containing the configuration information and the context on which configuration needs to be applied.
    
    :var configObj: Object holding the configuration information that needs to be applied
    :var context: The context to which the configuration needs to be applied.
    """
    configObj: BusinessObject = None
    context: BusinessObject = None


@dataclass
class CreateConfigInput(TcBaseObj):
    """
    Object containing input information for createOrUpdateConfigObjects action.
    
    :var data: CreateInput structure used to capture the inputs required for creation of a business object.
    :var modifyObject: Object to be modified.
    :var basedOn: Top line of the BOMWindow object in case of StructureContext or ConfigurationContext, Null in case of
    CollaborationContext.
    :var internalConfigData: Internal related config objects. For a CC, this would be a vector of SCs. For an SC, this
    would be a Config Context. For ConfigContext, this would be null.
    """
    data: CreateInput = None
    modifyObject: BusinessObject = None
    basedOn: BusinessObject = None
    internalConfigData: List[CreateConfigInput] = ()


@dataclass
class CreateConfigOutput(TcBaseObj):
    """
    Object containing information related to createOrUpdateConfigObjects action.
    
    :var object: Configuration object created or modified as per input.
    :var context: The context based on which configuration object is created.
    :var internalConfigData: Data regarding created and updated internal nested configuration objects.
    """
    object: BusinessObject = None
    context: BusinessObject = None
    internalConfigData: List[CreateConfigOutput] = ()


@dataclass
class CreateConfigResponse(TcBaseObj):
    """
    Response for createOrUpdateConfigObjects action.
    
    :var output: Object of CreateConfigOutput.
    :var serviceData: Service data including partial errors.
    """
    output: List[CreateConfigOutput] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.StructureManagement._2014_06.StructureVerification import ACFavoritesInput, ACFavoriteInfo, ACFavoritesResponse
from tcsoa.gen.BusinessObjects import BOMLine
from tcsoa.gen.StructureManagement._2014_06.StructureFilterWithExpand import ExpandAndSearchResponse, SearchCondition
from typing import List
from tcsoa.utils import TcService


class StructureVerificationService(TcService):

    @classmethod
    def manageACFavorites(cls, input: ACFavoritesInput) -> ACFavoritesResponse:
        """
        This operation creates, updates or deletes an accountability check favorite. Internally the dataset
        representing the accountablity check favorite is created, updated or deleted.
        To create accountability check favorite, the parameters account settings, name, description and action are
        mandatory. 
        To update or delete accountability check favorite, the parameters action and datasetUID are mandatory.
        """
        return cls.execute_soa_method(
            method_name='manageACFavorites',
            library='StructureManagement',
            service_date='2014_06',
            service_name='StructureVerification',
            params={'input': input},
            response_cls=ACFavoritesResponse,
        )

    @classmethod
    def getACFavorite(cls, datasetUID: str) -> ACFavoriteInfo:
        """
        This operation returns accountability check settings for a given dataset UID of accountability check favorite.
        These settings from a favorite are usually required to populate the accountability check dialog whenever a
        favorite is loaded or when accountability check is run.
        """
        return cls.execute_soa_method(
            method_name='getACFavorite',
            library='StructureManagement',
            service_date='2014_06',
            service_name='StructureVerification',
            params={'datasetUID': datasetUID},
            response_cls=ACFavoriteInfo,
        )


class StructureFilterWithExpandService(TcService):

    @classmethod
    def expandAndSearch(cls, lines: List[BOMLine], searchConditions: List[SearchCondition]) -> ExpandAndSearchResponse:
        """
        This operation does a full expansion of the given lines, then performs the search on the expanded structure
        with the given search conditions. The lines of a structure and search criteria as BOMLine property value are
        required. This operation returns the result BOMLines with satisfied condition indexes.
        A user can search for BOMLine with multiple search criteria by separating those criteria with an "OR" operator.
        The output line contains the information about the search conditions in the form of indexes which are satisfied
        by a particular BOMLine.
        Logical operator for the first condition is ignored
        This service will support search criterias like "find no = 10 and quantity = 1 or quantity = 20 and AbsOccId =
        CFG". In this criteria "AND" has precedence over "OR" operator.
        
        Comparison are done on the basis of BOMLine property type only.
        
        Valid inputs for Date type property : A user must give input in a specific date format that is defined in
        timelocal.xml If the date format is not defined in timelocal.xml then default format will be considered as
        "dd-mmm-yyyy hh:mm" .
        Example. "01-jan-2010 12:23"
        
        Invalid inputs for Date type property :    "01-jan-201012:23", "01-january-2010 12:23", "32-jan-2010 12:23",
        "32-jan-20112 12:23"
        
        
        Valid inputs for String values : A user must give a valid string for search it should not contain spaces until
        u meant to find a string with spaces. Leading and trailing spaces will be taken care.
        Example: "validInput" , "validInput  ", "   validInput", "  validInput   "
        Invalid Input : "valid   Input"
        Note: Since Comparison is done on the basis of BOMLine property type only. There are some integer and double
        type properties which is defined as string on BOMLine so you need to pass the exact value in that case that
        will be a string comparison.
        Example : "010" and "10" will be considered as different values
        
        Valid inputs for Boolean type property : only true and false is allowed. For those properties which is shown as
        Y and N in Rich Client you should pass the value as Y and N only.
        
        Valid inputs for Double and Integer type property : A user must provide a valid value which could be parsed
        successfully to specified type.
        Example : "12334" , "234.456", "007854", "0088.675"
        Invalid Input : "345fg", "fr4567", "456.54er"
        
        Note: 1) If a condition contains a invalid inputValue than comparison for that property will be skipped.
        Results will be returned for valid values only.
        2) Equal operator(=) will be used for wild card search by default. User must not pass a wild card character('*'
        or '?') in a string value. Search will be case insensitive for all relational operators except "==" operator.
        3) Only "AND" and "OR" logical operators are supported.
        4) For integer, double and Date types "=" and "==" operators have same behavior.
        
        
        Use cases:
        Search for BOMLine with their properties - A User can search for BOMLine in a collapsed structure by giving
        criteria as their property values like "bl_item_item_id = 000016" where bl_item_item_id is a property and
        000016 is input value given by user for search. 
        Example1: A User searches for some BOMLines by giving the criteria "find no. =10 OR quantity > 1" and BOMLine1
        satisfies the condition "find no. =10" and BOMLine2 satisfies the condition "quantity > 1" then the first
        output line will contain the BOMLine1 and search condition index  as 0 and the second output line will contain
        the BOMLine2 and search condition index as 1.
        
        Example2: A User searches for some BOMLines by giving the criteria "find no = 10 and quantity > 1 or Find No =
        20 and quantity = 1" and BOMLine1 satisfies the condition "find no = 10 and quantity > 1" and BOMLine2
        satisfies the condition "Find No = 20 and quantity = 1" then the first output line will contain the BOMLine1
        and search condition index  as 0,1 and the second output line will contain the BOMLine2 and search condition
        index as 
        """
        return cls.execute_soa_method(
            method_name='expandAndSearch',
            library='StructureManagement',
            service_date='2014_06',
            service_name='StructureFilterWithExpand',
            params={'lines': lines, 'searchConditions': searchConditions},
            response_cls=ExpandAndSearchResponse,
        )



from __future__ import annotations

from tcsoa.gen.Core._2013_05.DataManagement import GetChildrenResponse, GetPasteRelationsResponse, GenerateNextValuesIn, GetPasteRelationsInputData, SubTypeNamesResponse, SubTypeNamesInput, ValidateValuesResponse, GetChildrenInputData, ReviseIn, ReviseObjectsResponse, ValidateInput, GenerateNextValuesResponse
from tcsoa.gen.Core._2013_05.LOV import LOVData, LOVSearchResults, InitialLovData, LOVInput, ValidateLOVValueSelectionsResponse
from typing import List
from tcsoa.utils import TcService


class DataManagementService(TcService):

    @classmethod
    def getSubTypeNames(cls, inBOTypeNames: List[SubTypeNamesInput]) -> SubTypeNamesResponse:
        """
        This operation returns sub business object type names for each business object type name given as the input for
        the specified context.
        
        It returns the input base type in the sub business object type names list too.
        """
        return cls.execute_soa_method(
            method_name='getSubTypeNames',
            library='Core',
            service_date='2013_05',
            service_name='DataManagement',
            params={'inBOTypeNames': inBOTypeNames},
            response_cls=SubTypeNamesResponse,
        )

    @classmethod
    def reviseObjects(cls, reviseIn: List[ReviseIn]) -> ReviseObjectsResponse:
        """
        This operation is generic single revise operation for revisable business objects. This operation revises the
        given objects and copies or creates new objects using the data for the property values and deep copy data. Deep
        copy processing is recursive such that relations between secondary objects, or from secondary objects to the
        revised object, are replicated during this revise operation based upon deep copy rule configuration. This
        operation supports codeless configuration of custom properties. The following lists of revisable types are
        supported for this operation:
        - ItemRevision ( foundation template) and its sub-types
        - Identifier ( foundation template ) and its sub-types
        - Mdl0ConditionalElement (CPD solution ) and its sub-types
        
        """
        return cls.execute_soa_method(
            method_name='reviseObjects',
            library='Core',
            service_date='2013_05',
            service_name='DataManagement',
            params={'reviseIn': reviseIn},
            response_cls=ReviseObjectsResponse,
        )

    @classmethod
    def validateValues(cls, inputs: List[ValidateInput]) -> ValidateValuesResponse:
        """
        This' 'operation validates whether the input property values are valid according to defined naming rules and
        specified user exits for the input property.  Also, for the properties used in the multifield key (MFK)
        definition for the input type, this operation validates whether the combined property values makes up a unique
        value.
        
        The 'validateValues' operation can be called before other service operations that create new objects, such as
        the 'createObjects' or 'saveAsObjects' operations, in order to avoid errors that would occur during object
        creation due to invalid property values.  For example, if a large set of objects to be created is passed to the
        'createObjects' operation, where half of the set could fail due to invalid property values, the
        'validateValues' operation could be used to avoid those failures by checking whether the input object property
        values are valid.
        
        All of the input property names and their values should be included in 'ValidateInput' 'propValuesMap'.
        
        For user exit support, an existing user exit will be called if a specific property is specified along with
        additional input according to the user exit parameter names.  The specific property, referred to as the
        identifying property below, dictates which user exit will be called.  The additional input name and its value
        should also be include in 'ValidateInput' 'propValuesMap'.
        
        User exit name: 'USER_validate_dataset_name'
        Identifying property:
            'object_name': The Dataset name to be validated. This must be set in 'propValuesMap'
        Additional user exit parameter names:
            'vdnItemType': The Item type for the new Item, which has not necessarily been created yet.
            'vdnOldDataset': The old Dataset object for SaveAs.
            'vdnOldDatasetOwner': The old ItemRevision object.
            'vdnNewDatasetItemId': The Item ID for the new item, which has not necessarily been created yet.
            'vdnNewDatasetRevisionId': The ItemRevision ID for the new revision, which has not necessarily been created
        yet.
        
        Example 1:
        The following populated ValidateValuesInput will be used to call user exit for validating Dataset name:
        <ns1:ValidateValuesInput 
        <ns1:inputs clientId="42777::Dataset" operationType="0" businessObjectName=" Dataset ">
        <ns1:propValuesMap key="object_name"><ns1:value>DS003183</ns1:value></ns1:propValuesMap>
        </ns1:inputs></ns1:ValidateValuesInput>
        
        Example 2:
        The following populated ValidateValuesInput will be used to call user exit for validating Dataset name:
        <ns1:ValidateValuesInput 
        <ns1:inputs clientId="42777::Dataset" operationType="0" businessObjectName=" Dataset ">
        <ns1:propValuesMap key="object_name"><ns1:value>New DS</ns1:value></ns1:propValuesMap>
        <ns1:propValuesMap key="vdnItemType"><ns1:value>Item</ns1:value></ns1:propValuesMap>
        <ns1:propValuesMap key="vdnOldDataset"><ns1:value> SqYNv38Gx6sBFD </ns1:value> </ns1:propValuesMap>
        <ns1:propValuesMap key=" vdnOldDatasetOwner "><ns1:value></ns1:value></ns1:propValuesMap>
        <ns1:propValuesMap key=" vdnNewDatasetItemId "><ns1:value>003183</ns1:value> </ns1:propValuesMap>
        <ns1:propValuesMap key=" vdnNewDatasetRevisionId"><ns1:value>A</ns1:value> </ns1:propValuesMap>
        </ns1:inputs></ns1:ValidateValuesInput>
        
        User exit name: 'USER_validate_item_rev_id_3'
        The user exit will be called if the specified property is either 'item_id' or item_revision_id along with
        additional input according to the user exit parameter names.
        Identifying property:
            'item_id': The Item ID to be validated.  This must be set in 'propValuesMap'.
        Additional user exit parameter names:
            'item_revision_id': The ItemRevision ID to be validated.  This must be set in 'propValuesMap'.
            'object_type': The type of Item or ItemRevision for which the ID is being validated. If this property is
        not specified, system will use 'businessObjectName' in 'ValidateInput' as 'object_type'.  
            'viriItemObject': For Revise, this is ItemRevsion object&rsquo;s parent Item object. For Create and SaveAs,
        this is optional and can be empty value if specified.
        
        Example 1:
        The following populated ValidateValuesInput will be used to call user exit for validating Item ID:
        <ns1:ValidateValuesInput 
        <ns1:inputs clientId="42777::Item" operationType="0" businessObjectName="Item">
        <ns1:propValuesMap key="item_id"><ns1:value>003183</ns1:value></ns1:propValuesMap>
        </ns1:inputs></ns1:ValidateValuesInput>
        
        Example 2:
        The following populated ValidateValuesInput will be used to call user exit for validating ItemRevision ID:
        <ns1:ValidateValuesInput 
        <ns1:inputs clientId="42777::ItemRevision" operationType="1" businessObjectName="ItemRevision">
        <ns1:propValuesMap key="item_revision_id"><ns1:value>C</ns1:value></ns1:propValuesMap>
        <ns1:propValuesMap key=" viriItemObject "><ns1:value> SqYNv38Gx6sBFD </ns1:value></ns1:propValuesMap>
        </ns1:inputs></ns1:ValidateValuesInput>
        
        User exit name: 'USER_validate_alternate'
        Identifying property:
            'altid_of': The identifiable type object to which the alternate ID is associated.
        Additional user exit parameter names:
            'idcontext': The IDContext object which holds the cardinality rule for the given identifiable type object
        and the Identifier type object.
            'vaAltIdObject': The alternate ID object.
            'vaAltIdType': The Identifier type object.
        
        User exit name: 'USER_validate_alt_id'
        Identifying property:
            'idfr_id': The alternate ID which is to be validated against any of the naming rule associated with the
        Identifier type.
        Additional user exit parameter names:
            'idcontext': No longer used.
            'vaiIdfrType': The name of the Identifier type whose alternate ID is being validated.
            'vaiPatternName': The preferred name of the Identifier type or IdentifierRevision type wh ose alternate ID
        is being validated. Note:' USER_validate_alt_id' will choose 'pattern_name' as validating type if specified and
        will ignore 'idfr_type'.
        
        For the MFK uniqueness validation, it is important to note that the check for uniqueness is done against
        existing objects in Teamcenter and not between separate client inputs.  It is the responsibility of the client
        to validate whether separate input values in a single call would conflict with each other.
        
        The 'ValidateInput' 'operationType' input is currently a placeholder for future functionality that
        differentiates validation according to the workflow type, where validation for creating an object may be
        different then validation for saving an object.
        
        
        Use cases:
        Use Case 1:
        Prior to calling the service operation 'saveAsObjects', which may fail due to an error with any input property
        value, the client would call 'validateValues' for the properties to have Teamcenter check whether the values
        are valid.
        
        Use Case 2:
        Prior to making a call to the service operation 'createObjects' for several objects, the client wants to ensure
        that each object will be created with a unique identifier according to the MFK definition for the object type.
        The client would call 'validateValues' for the properties to have Teamcenter check that the values are valid
        and that the combined values would make a unique MFK value.
        """
        return cls.execute_soa_method(
            method_name='validateValues',
            library='Core',
            service_date='2013_05',
            service_name='DataManagement',
            params={'inputs': inputs},
            response_cls=ValidateValuesResponse,
        )

    @classmethod
    def generateNextValues(cls, generateNextValuesIn: List[GenerateNextValuesIn]) -> GenerateNextValuesResponse:
        """
        This operation generates values for the given properties of an object(s) during create/revise/save as action
        based on the user exits or naming rules configured on those properties. Customer user exits are given priority
        over the naming rules if both of them are configured on the same property. The counter has to be set active on
        the naming rule in order to generate the next value for a property. This operation also performs naming rule
        and multi field key validation on the generated values and return appropriate partial errors if the validation
        fails.
        This operation does not support generating values for attached Revision Name Rules on an Object type.
        For user exit support, an existing user exit will be called to generate values. Right now we support below
        given user exits for corresponding Object type.
        
        Object: Item
        User exit name: USER_new_item_id
        Property: item_id
        
        Object: ItemRevision
        User exit name: USER_new_revision_id, USER_new_revision_id_from_alt_rule (Baseline)
        Property: item_revision_id
        
        Object: Dataset
        User exit name: USER_new_dataset_id
        Property: pubr_object_id
        
        Object: Dataset
        User exit name: USER_new_dataset_rev
        Property: rev
        
        Object: Identifier
        User exit name: IDFR_new_alt_id, IDFR_new_rev_id (In Revise case)
        Property: idfr_id
        
        Object: CPD Objects
        User exit name: USER_new_cpd_id
        Property: CPD Objects related property
        
        Each of these user exits need some specific inputs which are required by them to generate IDs. These inputs are
        part of "generateNextValuesIn" structure and are described in details in its description.
        
        Use cases:
        a)    User clicks on assign button in Create/Revise/Saveas dialog:
        
        Autoassignable properties are those that have either a user exit or a naming rule with counter configured.A
        constant "autoassignable" is defined on the PropertyDescription class and its value can be obtained using
        PropertyDescription.getConstant() API. "Assign" button is displayed in create/revise/save as dialog to populate
        their values.
        
        This operation is used  to  generate the values for the autoassignable properties  when the user clicks on the
        "Assign" button.The caller should collect the list of all autoassignable properties  that do not have any user
        input and pass them to this operation. If  the autoassignable  property has a naming rule , the  naming rule
        pattern selected by the user  for  that property should also be passed as input to this operation. In all other
        cases the naming rule pattern should be passed as empty string.
        
        b)    User clicks on finish button in Create/Revise/Saveas dialog:
        
        This operation is also used to generate the values for the autoassignable properties  when the user clicks
        "Finish" button in in create/revise/save as dialog. The caller should collect the list of  all autoassignable
        properties that do not have any value generated and pass them to this operation. If  the autoassignable 
        property has a naming rule , the  naming rule pattern selected by the user  for  that property should also be
        passed as input to this operation. In all other cases the naming rule pattern should be passed as empty string.
        """
        return cls.execute_soa_method(
            method_name='generateNextValues',
            library='Core',
            service_date='2013_05',
            service_name='DataManagement',
            params={'generateNextValuesIn': generateNextValuesIn},
            response_cls=GenerateNextValuesResponse,
        )

    @classmethod
    def getChildren(cls, inputs: List[GetChildrenInputData]) -> GetChildrenResponse:
        """
        This operation returns the children for each input object.  The children returned is determined by the input
        'propertyNames' list of strings which defines the properties which are to be processed in order to collect the
        children to be returned  If the 'propertyNames' list is empty, then the properties which are processed to
        object the children is based on the <Type>_DefaultChildProperties and <Type>_DefaultPseudoFolder preferences. 
        Please see the Preferences and Environment Variables Reference and the RichClient Interface Guide for
        information on configuring these preferences. Children for which the user does not have read-access will be
        excluded from the returned list of children. No partial error is given if the 'propertyNames' list or the
        <Type>_DefaultChildProperties preference contains an invalid property name for the input object, children for
        the remaining 'propertyNames' will be returned.
        
        Use cases:
        Assume the following data exists in Teamcenter:
        Item
            Item Revision
            Item Master Form
        
        The ItemRevision exists in the Item's "revision_list" property.
        Item Item Master Form exists in the Item's "IMAN_master_form" property.
        
        Use case 1 (Get all children/no filter)
        1.    The user selects the Item in the above data in a tree viewer which shows all objects.
        2.    The user clicks the "+" to expand the Item.
        3.    The client then invokes getChildren with the selected object (Item), and no entries in the
        'propertyNames' argument.
        4.    getChildren determines the selected object's type, retrieves <Type>_DefaultChildProperties and
        <Type>_PseudoFolders preferences, and returns the Item Revision and Item Master Form, their type objects, and
        the propertyName in which they exist related to the parent.
        5.    The client then displays the returned list of children as child nodes in the tree.
        
        Use case 2 (Get subset of children/with filter)
        1.    The user selects the Item in the above data in a tree viewer which only shows object related via the
        revision_list property.
        2.    The user clicks the "+" to expand the Item.
        3.    The client then invokes getChildren with the selected object (Item), and gives "revision_list" in the
        'propertyNames' list.
        4.    getChildren iterates over the propertyNames list, and returns the Item Revision object, its child type
        object, and the propertyName in which it exists related to the parent.
        5.    The client then displays the returned list of children as child nodes in the tree.
        """
        return cls.execute_soa_method(
            method_name='getChildren',
            library='Core',
            service_date='2013_05',
            service_name='DataManagement',
            params={'inputs': inputs},
            response_cls=GetChildrenResponse,
        )

    @classmethod
    def getPasteRelations(cls, inputs: List[GetPasteRelationsInputData]) -> GetPasteRelationsResponse:
        """
        Returns the paste relation names for the given parent types and the child types, within which the expandable
        relations and the preferred paste relation are indicated.
        """
        return cls.execute_soa_method(
            method_name='getPasteRelations',
            library='Core',
            service_date='2013_05',
            service_name='DataManagement',
            params={'inputs': inputs},
            response_cls=GetPasteRelationsResponse,
        )


class LOVService(TcService):

    @classmethod
    def validateLOVValueSelections(cls, lovInput: LOVInput, propName: str, uidOfSelectedRows: List[str]) -> ValidateLOVValueSelectionsResponse:
        """
        This operation can be invoked after selecting a value from the LOV.  Use this operation to do additional
        validation to be done on server such as validating Range value, getting the dependent properties values in case
        of interdependent LOV (resetting the dependendent property values), Coordinated LOVs ( populating dependent
        property values )
        """
        return cls.execute_soa_method(
            method_name='validateLOVValueSelections',
            library='Core',
            service_date='2013_05',
            service_name='LOV',
            params={'lovInput': lovInput, 'propName': propName, 'uidOfSelectedRows': uidOfSelectedRows},
            response_cls=ValidateLOVValueSelectionsResponse,
        )

    @classmethod
    def getInitialLOVValues(cls, initialData: InitialLovData) -> LOVSearchResults:
        """
        This operation is invoked to query the data for a property having an LOV attachment. The results returned from
        the server also take into consideration any filter string that is in the input.  This operation returns both
        LOV meta data as necessary for the client to render the LOV and partial LOV values list as specified.
        
        The operation will return the results in the LOVSearchResults data structure. Maximum number of results to be
        returned are specified in the InitialLOVData data structure. If there are more results, the moreValuesExist
        flag in the LOVSearchResults data structure will be true. If the flag is true, more values can be retrieved
        with a call to the getNextLOVValues operation.
        """
        return cls.execute_soa_method(
            method_name='getInitialLOVValues',
            library='Core',
            service_date='2013_05',
            service_name='LOV',
            params={'initialData': initialData},
            response_cls=LOVSearchResults,
        )

    @classmethod
    def getNextLOVValues(cls, lovData: LOVData) -> LOVSearchResults:
        """
        This operation is invoked after a call to getInitialLOVValues if the moreValuesExist flag is true in the
        LOVSearchResults output returned from a call to the getInitialLOVValues operation. The operation will retrieve
        the next set of LOV values
        """
        return cls.execute_soa_method(
            method_name='getNextLOVValues',
            library='Core',
            service_date='2013_05',
            service_name='LOV',
            params={'lovData': lovData},
            response_cls=LOVSearchResults,
        )



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from tcsoa.gen.BusinessObjects import User, Fnd0CustomUserProfile
from dataclasses import dataclass
from typing import Dict, List


@dataclass
class UserProfileProperties(TcBaseObj):
    """
    User and its Fnd0CustomUserProfile properties
    
    :var user: User object on which Fnd0CustomUserProfile properties need to be set.
    :var propertyMap: Property map with property name and list of values in string format. The calling client is
    responsible for converting the different property types (int, float, date .etc) to a string using the appropriate
    functions in the SOA client framework's Property class.
    """
    user: User = None
    propertyMap: PropertyNameValuesMap = None


@dataclass
class UserProfilePropertiesResponse(TcBaseObj):
    """
    A list of Fnd0CustomUserProfile objects, one for each of the given User object.
    
    :var userProfileMap: User and its corresponding Fnd0CustomUserProfile object.
    :var serviceData: The Service Data with partial errors if any.
    """
    userProfileMap: UserProfileMap = None
    serviceData: ServiceData = None


"""
A map of property names and desired values (string/string).
"""
PropertyNameValuesMap = Dict[str, List[str]]


"""
A map of User and its Fnd0CustomUserProfile object.
"""
UserProfileMap = Dict[User, Fnd0CustomUserProfile]



from __future__ import annotations

from typing import List
from tcsoa.gen.Internal.Cad._2008_05.DataManagement import ResolveAttrMappingsForNXResponse
from tcsoa.gen.Cad._2008_03.DataManagement import ResolveAttrMappingsInfo
from tcsoa.utils import TcService


class DataManagementService(TcService):

    @classmethod
    def resolveAttrMappingsForNX(cls, info: List[ResolveAttrMappingsInfo]) -> ResolveAttrMappingsForNXResponse:
        """
        Retrieves NX specific CAD attribute mapped properties for one or more datasets. Information includes whether
        the property is frozen in the NX semantic. Also, an attribute can be a preference or constant, these may also
        be frozen. The client will have to look in the returned list to determine which one contains the property. The
        expected order of processing from the client is to iterate through the input list of 'clientIds'. If 'clientId'
        is present in the 'resolvedMappingsMap', then process the corresponding 'MappedDatasetAttrPropertyInfo'. If it
        is not in the 'resolvedMappingsMap', check if the 'clientId' is present in the
        'resolvedConstOrPrefAttrInfoMap'. If the attribute mapping is of the ATTRMAP_preference or ATTRMAP_constant
        type, the mapped value may be found in the 'resolvedConstOrPrefAttrInfoMap'.
        
        Use cases:
        NX Client needs to know if an attribute is mapped and if mapped if it modifiable.
        """
        return cls.execute_soa_method(
            method_name='resolveAttrMappingsForNX',
            library='Internal-Cad',
            service_date='2008_05',
            service_name='DataManagement',
            params={'info': info},
            response_cls=ResolveAttrMappingsForNXResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Role, ClosureRule, Fnd0SubscriptionTemplate, BusinessObject, Group, ImanSubscription
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetAttachEventDataResponse2(TcBaseObj):
    """
    The GetAttachEventDataResponse2 structure represents the output response returning lists of relation objects and
    attachment types. Partial errors will be returned and wrapped in serviceData, if any are specified.
    
    :var relations: A list of relation objects.
    :var attachmentTypes: A list of attachment type business objects.
    :var serviceData: Partial errors will be returned and wrapped in serviceData, if any are specified.
    """
    relations: List[BusinessObject] = ()
    attachmentTypes: List[BusinessObject] = ()
    serviceData: ServiceData = None


@dataclass
class AttributeCriteria2(TcBaseObj):
    """
    The AttributeCriteria2 used in Fnd0IndirectSubscription object through  Fnd0SubscriptionTemplate object adds
    additional condition as to how a Subscription should be processed. It comprises of logical operators like "AND" or
    "OR" and math operators like "=, !=, <, >, <=, >=". The attribute name and attribute values are property names and
    their applicable values respectively.
    
    :var logicOperators: A list of logic operators
    :var attributeNames: A list of attribute names
    :var mathOperators: A list of math operators
    :var attributeValues: A list of attribute values
    """
    logicOperators: List[str] = ()
    attributeNames: List[str] = ()
    mathOperators: List[str] = ()
    attributeValues: List[str] = ()


@dataclass
class ModifySubscriptionTemplateInput(TcBaseObj):
    """
    The ModifySubscriptionTemplateInput structure holds the data required to modify a Fnd0SubscriptionTemplate object
    
    :var subscriptionTemplate: The Fnd0SubscriptionTemplate object to be modified
    :var templateData: The SubscriptionTemplate structure holds the data required to modify a Fnd0SubscriptionTemplate
    object
    """
    subscriptionTemplate: Fnd0SubscriptionTemplate = None
    templateData: SubscriptionTemplate = None


@dataclass
class SaveAsSubscriptionInputData(TcBaseObj):
    """
    A structure used by saveAsSubscription operation to hold the subscription to be copied and a new name to be given
    to the subscription to be created.
    
    :var subscription: The ImanSubscription to be copied.
    :var name: The name to be given to the new subscription.
    """
    subscription: ImanSubscription = None
    name: str = ''


@dataclass
class SubscriptionTemplate(TcBaseObj):
    """
    The SubscriptionTemplate structure holds the data required to create an Fnd0SubscriptionTemplate object
    
    :var name: Name of the template
    :var description: Description of the template
    :var subscribedObjectType: Subscribed object type which can be any object under WorkspaceObject
    :var subscribedObjectCriteria: A structure to hold the attribute criteria defined on the subscribed object
    :var targetObjectType: Target object type selected, must be a subscribable business object and can be any object
    under WorkspaceObject
    :var targetObjectCriteria: A structure to hold the attribute criteria defined on the target object
    :var closureRule: The closure rule to be used for traversal in the template
    :var validAccessors: The Group and Role objects with an access to the Fnd0SubscriptionTemplate object
    """
    name: str = ''
    description: str = ''
    subscribedObjectType: BusinessObject = None
    subscribedObjectCriteria: AttributeCriteria2 = None
    targetObjectType: BusinessObject = None
    targetObjectCriteria: AttributeCriteria2 = None
    closureRule: ClosureRule = None
    validAccessors: Accessors = None


@dataclass
class SubscriptionTemplateResponse(TcBaseObj):
    """
    A structure to hold the Fnd0SubscriptionTemplate object created or modified through an operation
    
    :var subscriptionTemplates: A list of Fnd0SubscriptionTemplate objects to be returned
    :var serviceData: Partial errors will be returned and wrapped in serviceData, if any are specified.
    """
    subscriptionTemplates: List[Fnd0SubscriptionTemplate] = ()
    serviceData: ServiceData = None


@dataclass
class Accessors(TcBaseObj):
    """
    A structure to hold the lists of Group and Role objects that have an access to the Fnd0SubscriptionTemplate object
    A user who belongs to the listed groups and roles will have an access to view or modify the subscription templates.
    
    :var groups: A list of Group objects with an access to the Fnd0SubscriptionTemplate object
    :var roles: A list of Role objects with an access to the Fnd0SubscriptionTemplate object
    """
    groups: List[Group] = ()
    roles: List[Role] = ()



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExportObjectsToPLMXMLResponse(TcBaseObj):
    """
    The response for 'exportObjectsToPLMXML' operation. It holds the file ticket for the exported XML file, file ticket
    for the export log file, file tickets  for the dataset named reference files, and any partial failures.
    
    :var xmlFileTicket: The FMS ticket is used to get the generated PLMXML file.
    :var logFileTicket: The FMS ticket is used to get the generated export log file.
    :var namedRefFileTickets: The FMS tickets are used to get the dataset named  reference files. On Teamcenter
    Services client, the files must be loaded into the directory along with the PLMXML file. And the directory must
    have the same name as the PLMXML file without the file extension name.
    :var serviceData: Service data contains the list of created or modified objects and also the partial errors is used
    to report any partial failures.
    """
    xmlFileTicket: FileTicket = None
    logFileTicket: FileTicket = None
    namedRefFileTickets: List[FileTicket] = ()
    serviceData: ServiceData = None


@dataclass
class FileTicket(TcBaseObj):
    """
    To represent a  file ticket and its original file name.
    
    :var ticket: The FMS file Ticket.
    :var fileName: The original file name.
    """
    ticket: str = ''
    fileName: str = ''


@dataclass
class ImportObjectsFromPLMXMLResponse(TcBaseObj):
    """
    The response for 'importObjectsFromPLMXML' operation. It holds the file ticket for the import log file, and any
    partial failures.
    
    :var logFileTicket: The FMS ticket is used to get the generated import log file.
    :var serviceData: Service data contains the list of created or modified objects and also the partial errors.
    """
    logFileTicket: FileTicket = None
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.DocumentManagement._2010_04.DigitalSignature import DigtalSigningSaveResponse, DigitalSignSaveInput
from tcsoa.gen.DocumentManagement._2010_04.LaunchDefinition import ServerInfo, SessionInfo, LaunchDefinitionResponse, UserAgentDataInfo, LDSelectedInputInfo
from typing import List
from tcsoa.utils import TcService


class DigitalSignatureService(TcService):

    @classmethod
    def digitalSigningSave(cls, saveInput: DigitalSignSaveInput) -> DigtalSigningSaveResponse:
        """
        The digitalSigningSave function will update an existing dataset that has the name referenced uploaded signed
        file.
        """
        return cls.execute_soa_method(
            method_name='digitalSigningSave',
            library='DocumentManagement',
            service_date='2010_04',
            service_name='DigitalSignature',
            params={'saveInput': saveInput},
            response_cls=DigtalSigningSaveResponse,
        )


class LaunchDefinitionService(TcService):

    @classmethod
    def getLaunchDefinition(cls, operation: str, selectedInputs: List[LDSelectedInputInfo], serverInfo: ServerInfo, sessionInfo: SessionInfo, userAgentData: UserAgentDataInfo) -> LaunchDefinitionResponse:
        """
        The Application Launcher (AppLauncher) uses a launch definition XML as input to launch appropriate external
        applications. This operation gathers the data and builds the launch definition XML string. It contains
        information for list of supported tools, business data and tool preferences. The definition XML is based on the
        list of 'LDSelectedInputInfo' structure ( contains WorkspaceObject, related Item, related ItemRevision, related
        control WorkspaceObject business object, request mode and additional information in the form of key value pair
        strings), structure of server information 'ServerInfo' where the operation is initiated, structure of the
        session information 'SessionInfo' of the client from where the operation is initiated, and structure of client
        information 'UserAgentDataInfo' from where the operation is initiated.
        The required input data from the 'LDSelectedInputInfo' structure is the WorkspaceObject business object
        (normally this input is the subtype of WorkspaceObject business object such as Item business object or
        ItemRevision business object or Dataset business object).    The input structures for server, session, and
        client information can be empty.
        
        Use cases:
        Use case1:  View/Markup action from client
        When a user selects an Item or an ItemRevision or a Dataset and performs View/Markup action in the client, the
        system will invoke the getLaunchDefinition operation.
        Use case2:  Office client open
        When a user performs open action on an MSWord Dataset and the client is configured to use AppLauncher for open,
        the system will invoke the getLaunchDefinition operation.
        Use case3: Active Workspace Office client open
        When a user performs open and edit in office client action on an MSWord Dataset and the client is configured to
        use AppLauncher for open, the system will invoke the getLaunchDefinition operation.
        
        """
        return cls.execute_soa_method(
            method_name='getLaunchDefinition',
            library='DocumentManagement',
            service_date='2010_04',
            service_name='LaunchDefinition',
            params={'operation': operation, 'selectedInputs': selectedInputs, 'serverInfo': serverInfo, 'sessionInfo': sessionInfo, 'userAgentData': userAgentData},
            response_cls=LaunchDefinitionResponse,
        )



from __future__ import annotations

from tcsoa.gen.Ai._2006_03.Ai import ApplicationRef
from enum import Enum
from typing import List
from tcsoa.gen.Ai._2008_06.Ai import ObjectsWithConfig
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetPropertyValuesData(TcBaseObj):
    """
    Used to input the list of objects with related configuration (used to setup bomwindows if needed on server), and
    the properties.
    
    :var objs: list of objects along with configuration.
    :var properties: list of  properties to be queried for on the object.
    """
    objs: List[ObjectsWithConfig] = ()
    properties: List[str] = ()


@dataclass
class GetPropertyValuesResponse(TcBaseObj):
    """
    capture the property values for the specified object and specified properties and any failures.
    
    :var objProps: the properties of the object
    :var failedSetIndices: array of failed indices. The index is the position in the input array. And the indices
    member is the list of failed objects (invalid tags) at each such index.
    """
    objProps: List[ObjPropDetail] = ()
    failedSetIndices: List[ErrorMap] = ()


@dataclass
class ObjPropDetail(TcBaseObj):
    """
    capture the object ApplicationRef and it's property details.
    
    :var obj: the ApplicationReference of the obj (uid/appname/version)
    :var properties: array or property details per
    :var failedPropIndices: index of the failed property for the object - the index maps to the input property array.
    :var failedPropMessages: the error string corresponding to the error id in failedPropIndices.
    """
    obj: ApplicationRef = None
    properties: List[PropertyDetails] = ()
    failedPropIndices: List[int] = ()
    failedPropMessages: List[str] = ()


@dataclass
class PropertyDetails(TcBaseObj):
    """
    details of a property
    
    :var name: display name of the property
    :var values: values in string form. Array - if the property has multiple values. These values can be decoded/parsed
    using the Property SOA client class if needed.
    :var maxStrLen: in case the property value is  a string - the maximum possible length.
    :var type: the type of the property as a string.
    integer = "int", short="short", float="float", double="double", char="char", logical="logical",note="note",
    string="string", date="date", any reference="reference"
    :var access: will be set to 0 if write access is allowed, 1 - for read.
    :var usage: if lov - the usage type of that lov.
    :var lovValues: the lovValues as strings.
    :var nullElement: for each value in an array - is the value Null.
    :var emptyElement: Used to indicate if each element in an array(each value) is empty.
    """
    name: str = ''
    values: List[str] = ()
    maxStrLen: int = 0
    type: str = ''
    access: int = 0
    usage: LOVUSAGE = None
    lovValues: List[str] = ()
    nullElement: List[int] = ()
    emptyElement: List[int] = ()


@dataclass
class ErrorMap(TcBaseObj):
    """
    capture the index of the input array, and the indices for the objects within that array.
    
    :var index: index of the input array with the list of objects and configurations.
    :var indices: Within the index specified by "index" the location of failed indices.
    """
    index: int = 0
    indices: List[int] = ()


class LOVUSAGE(Enum):
    """
    The different kinds of lov usage
    """
    AI_LV_Unknown = 'AI_LV_Unknown'
    AI_LV_Exhaustive_list = 'AI_LV_Exhaustive_list'
    AI_LV_Suggestions = 'AI_LV_Suggestions'
    AI_LV_Ranges = 'AI_LV_Ranges'
    AI_LV_Upper_bound = 'AI_LV_Upper_bound'



from __future__ import annotations

from tcsoa.gen.AuthorizedDataAccess._2017_05.LicenseManagement import LicenseAttachOrDetachInput
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class LicenseManagementService(TcService):

    @classmethod
    def attachOrDetachLicensesFromObjects(cls, licenseAttachOrDetachInput: List[LicenseAttachOrDetachInput]) -> ServiceData:
        """
        This operation attaches or detaches ADA_License objects to WorkspaceObject objects such as Item, ItemRevision,
        Dataset, etc. as specified in each LicenseAttachOrDetachInput. Optionally, this operation can set/update
        authorizing paragraph information for the ITAR licenses being attached. Users performing this operation will
        need IP_ADMIN privilege to both the workspace objects and the licenses being attached, if the licenses are of
        type IP or Exclude, while ITAR_ADMIN privilege is needed to both the workspace objects and the licenses being
        attached, if the licenses are of ITAR license type. If the user does not have necessary privilege to attach any
        licenses, or if there is any other error while attaching licenses, the errors are returned as partial errors in
        ServiceData.
        
        Use cases:
        - You can attach ADA_License business objects to classified WorkspaceObject business objects to grant access
        for users and/or groups listed on the licenses in context of structure configuration.
        - You can remove ADA_License business objects from classified WorkspaceObject business objects to revoke access
        for users and/or groups listed on the licenses in context of structure configuration.
        
        """
        return cls.execute_soa_method(
            method_name='attachOrDetachLicensesFromObjects',
            library='AuthorizedDataAccess',
            service_date='2017_05',
            service_name='LicenseManagement',
            params={'licenseAttachOrDetachInput': licenseAttachOrDetachInput},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, WorkspaceObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class AttributeFormat(TcBaseObj):
    """
    Structure representing format details of a classification attribute.
    
    :var formatType: Integer representing the attribute type, which could be one of the following:
    - - 1 => KeyLOV 
    - 0 => String 
    - 1 => Integer 
    - 2 => Real 
    - 3 => Date 
    - 4 => Time range.
    
    
    :var formatModifier1: Integer to indicate whether the attribute is configured for one of the following - 
    - 0 - Force positive number.
    - 1 - Accept and display either + or - 
    - 2 - Accept + or - but display - sign only.
    
    
    
    Note: This field will return 0 if not applicable. Only applicable if the selected formatType is 1 or 2.
    :var formatModifier2: Integer to indicate additional information about the selected formatType from one of the
    following : 
    If formatType = 0 then:
    - 0 = Print characters in upper and lower case.
    - 1 = Print all characters in upper case.
    - 2 = Print all characters in lower case.
    
    
    
    If formatType == 2 then Number of digits after the decimal point
    
    Note: This field will return 0 if not applicable. Only applicable if the selected formatType is 0 or 2.
    :var formatLength: Integer representing the length of the attribute. In case of a KeyLOV (stxt), this will contain
    the KeyLOV ID.
    """
    formatType: int = 0
    formatModifier1: int = 0
    formatModifier2: int = 0
    formatLength: int = 0


@dataclass
class ClassDocuments(TcBaseObj):
    """
    Structure holding the attached Icon, Image or NamedRef or this child class.
    
    :var documentType: Contains type of document attached. Valid values are:
    - icon 
    - image 
    - NamedRef
    
    
    :var ticket: File ticket identifier for the attached file.
    :var originalFileName: File name for this attachment.
    """
    documentType: str = ''
    ticket: str = ''
    originalFileName: str = ''


@dataclass
class ClassParents(TcBaseObj):
    """
    Structure representing classification class' parents details.
    
    :var parents: A list of class parents' details.
    """
    parents: List[ClassDefinition] = ()


@dataclass
class ClassSearchCriteria(TcBaseObj):
    """
    A structure defining the search criteria used for searching classes from classification hierarchy.  Also allows
    sorting the results based on a predefined criterion.
    
    :var searchAttribute: Class attribute to be searched for. Valid values are all class properties like class Id,
    class Name etc.
    :var searchString: Query string to search the class attribute by.
    :var sortOption: Option to sort the returned results. Valid values are the class properties like class Id, class
    Name etc.
    """
    searchAttribute: int = 0
    searchString: str = ''
    sortOption: int = 0


@dataclass
class ClassificationObjectInfo(TcBaseObj):
    """
    Structure representing Classification Object details.
    
    :var clsObject: The found Classification object.
    :var workspaceObject: The WorkspaceObject (WSO) that is associated by this Classification object. If this is
    NULLTAG, then the found "clsObject" is a standalone classification object (ICO). Allowed WSO types will be defined
    by the preference 'ICS_allowable_types'.
    :var properties: List of properties containing, attribute Ids and their values.
    """
    clsObject: BusinessObject = None
    workspaceObject: WorkspaceObject = None
    properties: List[ClsProperty] = ()


@dataclass
class ClassificationObjects(TcBaseObj):
    """
    A structure representing list of classification objects information.
    
    :var clsObjects: A list of classification object's information.
    """
    clsObjects: List[ClassificationObjectInfo] = ()


@dataclass
class ClsPropValue(TcBaseObj):
    """
    Structure containing classification property value details.
    
    :var internalValue: Internal value stored into database.
    :var displayValue: Display value for a classification property. This will be same as that of internalValue for all
    classification attributes except keyLOV (stxt) attributes as they will have "entry key" as internalValue and "entry
    key entry value" or "entry value" as displayValue; based on the keyLOV configuration.
    """
    internalValue: str = ''
    displayValue: str = ''


@dataclass
class ClsProperty(TcBaseObj):
    """
    Structure representing Classification Property which holds attribute ids and their values.
    
    :var propertyId: The unique identifier of classification attribute or internal identifier for classification header
    properties like class Id, unit system etc.
    :var propertyName: The name of classification attribute or name of internal identifier for classification header
    properties like class Id, unit system etc.
    :var values: A list of values for this attribute.
    [Note: An array is required as an attribute can be single or multi-valued.]
    """
    propertyId: int = 0
    propertyName: str = ''
    values: List[ClsPropValue] = ()


@dataclass
class ExtendedProperties(TcBaseObj):
    """
    Structure representing classification attributes' extended metadata properties details.
    
    :var propName: Extended metadata property name.
    :var propValues: A list of extended metadata property values.
    """
    propName: str = ''
    propValues: List[ClsPropValue] = ()


@dataclass
class FindClassificationsResponse(TcBaseObj):
    """
    Structure representing the classification information returned by FindClassifications operation.
    
    :var clsObjectDefMap: A map (WorkspaceObject/ClassificationObjectInfo) of workspace objects to a list of
    Classification objects.
    :var keyLOVDefMap: A map (string/KeyLOVDefinition) of Classification KeyLOV IDs and its definitions pairs .
    :var clsAttrdefMap: A map (string/ClassAttributesDefinition) of Classification class or view IDs and its definition
    pairs .
    :var clsDefMap: A map (string/ClassDefinition) of Classification class or view IDs and its corresponding definition
    pairs.
    :var clsParentsMap: A map (string/ClassParents) of Classification classes or view IDs and its parents' definition
    pairs.
    :var clsChildrenMap: A map (string/ClassChildren) of Classification classes or view IDs and its children's
    definition pairs.
    :var serviceData: Any failures will be returned in the service data list of partial errors.
    """
    clsObjectDefMap: ClsObjectsDefinitionMap = None
    keyLOVDefMap: KeyLOVDefinitionMap = None
    clsAttrdefMap: ClassAttributeDefinitionMap = None
    clsDefMap: ClassDefinitionMap = None
    clsParentsMap: ClassParentsMap = None
    clsChildrenMap: ClassChildrenMap = None
    serviceData: ServiceData = None


@dataclass
class FormatProperties(TcBaseObj):
    """
    Structure representing format details.
    
    :var formatDefinition: Attribute format definition.
    :var unitName: Unit display name associated with this attribute in this unit system.
    :var defaultValue: Default value of this Class attribute. This can be an empty string indicating no default value.
    :var minimumValue: Minimum value constraint of this Class attribute. This can be an empty string indicating no
    minimum value constraint.
    Note: Only applicable to numerical formats of attributes.
    :var maximumValue: Maximum value constraint of this Class attribute. This can be an empty string indicating no
    maximum value constraint.
    Note: Only applicable to numerical formats of attributes.
    """
    formatDefinition: AttributeFormat = None
    unitName: str = ''
    defaultValue: str = ''
    minimumValue: str = ''
    maximumValue: str = ''


@dataclass
class ClassAttribute(TcBaseObj):
    """
    Structure representing class attribute details.
    
    :var attributeId: Classification attribute Id.
    :var metricFormat: Attribute format definition in metric unit system.
    :var nonMetricFormat: Attribute format definition in non-metric unit system.
    :var arraySize: Array size or the number of values for this attribute.
    - If single valued (nonarray), then arraySize = 1 
    - If multi valued (array), then arraySize >= 1 corresponding to the size of the array defined in the attribute
    definition.
    
    
    :var options: Attribute property flags represented as a single integer. To access individual property flags, a
    bitwise operation will be required by the client.Valid values are:
    - ATTR_vla          = (1 << 0) 
    - ATTR_external_vla = (1 << 1) 
    - ATTR_mandatory     = (1 << 2) 
    - ATTR_protected     = (1 << 3) 
    - ATTR_unique         = (1 << 4) 
    - ATTR_propagated     = (1 << 5) 
    - ATTR_localValue     = (1 << 6) 
    - ATTR_reference     = (1 << 7) 
    - ATTR_auto_computed = (1 << 15) 
    - ATTR_hidden         = (1 << 20) 
    - ATTR_localizable = ( 1 << 22 )
    
    
    :var extendedProperties: A list of classification class attributes' extended metadata properties.
    :var attributeProperties: A list of classification attributes' property values. These properties could be attribute
    Id, attribute Name, attribute shortName etc.
    """
    attributeId: int = 0
    metricFormat: FormatProperties = None
    nonMetricFormat: FormatProperties = None
    arraySize: int = 0
    options: int = 0
    extendedProperties: List[ExtendedProperties] = ()
    attributeProperties: List[ClsProperty] = ()


@dataclass
class KeyLOVDefinition(TcBaseObj):
    """
    Structure representing KeyLOV (stxt) definition.
    
    :var keyLOVOptions: KeyLOV (stxt) options to Show/Hide keys. Valid values are: 
    - 0 = Show key 
    - 1 = Hide key
    
    
    :var keyLOVEntries: List of KeyLOV (stxt) entries.
    :var owningSite: Owning Site (POM_imc) of this keyLOV (stxt) object.
    :var sharedSites: List of sites (POM_imc) where this KeyLOV (stxt) is shared using Multisite operations.
    :var keyLOVProperties: keyLOV (stxt) properties like keyLOV Id, name etc. details.
    """
    keyLOVOptions: int = 0
    keyLOVEntries: List[KeyLOVEntry] = ()
    owningSite: BusinessObject = None
    sharedSites: List[BusinessObject] = ()
    keyLOVProperties: List[ClsProperty] = ()


@dataclass
class KeyLOVEntry(TcBaseObj):
    """
    Structure representing KeyLOV (stxt) Entry.
    
    :var keyLOVkey: String representing a Key of a KeyLOV (stxt) entry.
    :var keyLOVValue: String representing a Value of the KeyLOV (stxt) entry.
    :var isDeprecated: Flag indicating whether this KeyLOV (stxt) entry is deprecated.
    If true, keyLOV entry is deprecated and can not be used for new classifications otherwise, it can be used for new
    classifications.
    """
    keyLOVkey: str = ''
    keyLOVValue: str = ''
    isDeprecated: bool = False


@dataclass
class ClassAttributesDefinition(TcBaseObj):
    """
    The structure containing list of Classification class attributes definition and configured KeyLOV (stxt) definition.
    
    :var classAttributes: The list of attributes defined for the classification class.
    :var configKeyLOVMap: A map (int/KeyLOVDefinition) of attribute ID and KeyLOV (stxt) definition pairs, based on
    dependency configuration of an attribute.
    """
    classAttributes: List[ClassAttribute] = ()
    configKeyLOVMap: ConfiguredKeyLOVDefinitionMap = None


@dataclass
class ClassChildren(TcBaseObj):
    """
    Structure representing classification class' children details.
    
    :var children: A list of class' children details.
    """
    children: List[ClassDefinition] = ()


@dataclass
class ClassDefinition(TcBaseObj):
    """
    Structure holding properties information about the given class.
    
    :var childCount: Number of child classes for this class.
    :var instanceCount: Total number of classification objects instantiated in this class or any of its descendants.
    :var viewCount: Number of Views defined for this class.
    :var options: Property describing classification class' flags represented as a single integer.
    To access individual property flags, a bitwise operation will be required by the client. Valid values are:
    - CLASS_falg_englishUnitBase = (1 << 0 )
    - CLASS_flag_isGroup          = (1 << 1)
    - CLASS_flag_unit_system_both = (1 << 2)
    - CLASS_flag_isStorrageClass  = (1 << 4)
    - CLASS_flag_isAssembly       = (1 << 5)
    
    
    :var documents: A list of attached Icons, Images and NamedRefs to this class.
    :var properties: A list of classification class properties like class Id, Name, ShortName etc. details.
    """
    childCount: int = 0
    instanceCount: int = 0
    viewCount: int = 0
    options: int = 0
    documents: List[ClassDocuments] = ()
    properties: List[ClsProperty] = ()


"""
The list of Classification Class or View ID and corresponding definition.
"""
ClassDefinitionMap = Dict[str, ClassDefinition]


"""
A map of classification class Id to a list of its parents.
"""
ClassParentsMap = Dict[str, ClassParents]


"""
Map of workspace object to a list of its classification objects. If an object is classified then map value will have all its classification's information. If object is not classified but system found standalone classification objects then the map values will have standalone classification object's information.
"""
ClsObjectsDefinitionMap = Dict[WorkspaceObject, ClassificationObjects]


"""
A list of attribute ID and corresponding configured KeyLOV (stxt) definition based on dependency configuration set on class attribute.
"""
ConfiguredKeyLOVDefinitionMap = Dict[int, KeyLOVDefinition]


"""
The list of KeyLOV (stxt) id and corresponding KeyLOV (stxt) definition.
"""
KeyLOVDefinitionMap = Dict[str, KeyLOVDefinition]


"""
The list of Classification Class or View ID and corresponding attributes definition.
"""
ClassAttributeDefinitionMap = Dict[str, ClassAttributesDefinition]


"""
A map of classification class Id to a list of its children.
"""
ClassChildrenMap = Dict[str, ClassChildren]



from __future__ import annotations

from tcsoa.gen.Internal.OccMgmt._2020_05.ImportExport import ImportExcelResponse, MappingGroupInput, MappingGroupResponse, ImportExcelData
from typing import List
from tcsoa.utils import TcService


class ImportExportService(TcService):

    @classmethod
    def importExcelAndUpdateMappingGrp(cls, importExcelData: List[ImportExcelData]) -> ImportExcelResponse:
        """
        Imports an Excel file into Teamcenter and creates the product structure and sets the properties on objects
        based on the values given in Excel file. The input Excel file contains a level column to indicate the level of
        the object in the structure. "Level" and "Object Type" columns are mandatory in the input Excel sheet. The
        input Excel file is parsed to import the data to create product structure in Teamcenter. This operation
        supports the creation of objects only. Update and Delete are not supported. The column headers in excel are
        mapped to their respective properties which are saved as a mapping group on "import". This operation support
        create and update of mapping groups.
        
        Use cases:
        1.    The user creates a new mapping group by mapping the property names with the column headers in the Excel
        sheet, giving a new group name and clicking on "Import" button.
        2.    The user uses an existing mapping group by selecting a group name which populates the existing mappings
        and clicking on "Import" button.
        3.    The user updates an existing mapping group by selecting a group name, changing some of the existing
        mappings and clicking on "Import" button.
        """
        return cls.execute_soa_method(
            method_name='importExcelAndUpdateMappingGrp',
            library='Internal-OccMgmt',
            service_date='2020_05',
            service_name='ImportExport',
            params={'importExcelData': importExcelData},
            response_cls=ImportExcelResponse,
        )

    @classmethod
    def importExcelAndUpdateMappingGrpAsync(cls, importExcelData: List[ImportExcelData]) -> None:
        """
        Imports an Excel file into Teamcenter and creates the product structure and sets the properties on objects
        based on the values given in Excel file. The input Excel file contains a level column to indicate the level of
        the object in the structure." Level" and "Object Type" columns are mandatory in the input Excel sheet. The
        input Excel file is parsed to import the data to create product structure in Teamcenter. This operation
        supports the creation of objects only. Update and Delete are not supported. The column headers in excel are
        mapped to their respective properties which are saved as a mapping group on "import". This operation support
        create and update of mapping groups.
        
        Use cases:
        1.    The user creates a new mapping group by mapping the property names with the column headers in the Excel
        sheet, giving a new group name and clicking on "Import" button.
        2.    The user uses an existing mapping group by selecting a group name which populates the existing mappings
        and clicking on "Import" button.
        3.    The user updates an existing mapping group by selecting a group name, changing some of the existing
        mappings and clicking on "Import" button.
        """
        return cls.execute_soa_method(
            method_name='importExcelAndUpdateMappingGrpAsync',
            library='Internal-OccMgmt',
            service_date='2020_05',
            service_name='ImportExport',
            params={'importExcelData': importExcelData},
            response_cls=None,
        )

    @classmethod
    def getMappingGroupInfo(cls, inputs: List[MappingGroupInput]) -> MappingGroupResponse:
        """
        Returns all the mapping group information which includes the names of all mapping groups as well as the mapping
        information of each group. The input Excel file contains a level column to indicate the level of the object in
        the structure. "Level" and "Object Type" columns are mandatory in the input Excel sheet. The column headers of
        the excel file are mapped to their respective properties which are saved into a mapping group. The user can
        create, update or delete the mapping group. The input Excel file is parsed to import the data to create product
        structure in Teamcenter.
        
        Use cases:
        1) User clicks on the "Browse" button to parse an Excel file with a header row that contains display names of
        the properties. This header information is parsed and user is presented mapping screens to map the property
        names with the column headers in the Excel sheet.
        2) If the user selects an existing saved mapping group, the mapped properties get populated for the respective
        column headers.
        """
        return cls.execute_soa_method(
            method_name='getMappingGroupInfo',
            library='Internal-OccMgmt',
            service_date='2020_05',
            service_name='ImportExport',
            params={'inputs': inputs},
            response_cls=MappingGroupResponse,
        )



from __future__ import annotations

from tcsoa.gen.Core._2008_06.DataManagement import BOWithExclusionIn
from tcsoa.gen.Core._2007_06.DataManagement import BaseClassInput
from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Core._2010_04.DataManagement import LocalizedPropertyValuesList, GetDatasetCreationRelatedInfoResponse2, PropertyInfo, DisplayableSubBusinessObjectsResponse, CreateDatasetsResponse, LocalizableStatusResponse, LocalizedPropertyValuesInfo, LocalizableStatusInput, DatasetInfo, GetAvailableBusinessObjectTypesResponse
from tcsoa.gen.Core._2010_04.Session import GetShortcutsResponse, LHNShortcutInputs, MultiPreferenceResponse2
from tcsoa.gen.Core._2010_04.LanguageInformation import TranslationStatusResponse, LanguageResponse
from typing import List
from tcsoa.gen.Core._2007_01.Session import ScopedPreferenceNames
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class SessionService(TcService):

    @classmethod
    def getShortcuts(cls, shortcutInputs: LHNShortcutInputs) -> GetShortcutsResponse:
        """
        This operation gets the sections and corresponding content in Left Hand Navigation task pane given the section
        name and the corresponding preference name for the current session user. The preference name is the key to look
        up section content stored in preference.    In the rich client, the LHN sections are Quick Links, Open Items,
        History, Favorites and I Want To. The user can organize Teamcenter data in these sections during runtime, which
        is persisted in the preference. The Quick Links section provides a quick access point to the user`s home
        folder, work list, favorite Web links, projects, saved searches, and View Markup. The Open Items section lists
        Teamcenter components currently opened in the active perspective. The History section lists Teamcenter
        components opened before, but currently closed. The Favorites section contains the Favorites container and
        Teamcenter components the user added there for quick access. The I Want To section contains commands configured
        by default or configured by the user.
        
        Use cases:
        The user logs in to the rich client and retrieves Quick Links, Open Items, History, Favorites and I Want To
        task pane section for Left Hand Navigation.
        
        Exceptions:
        >Service Exception    Thrown if there is an empty or invalid section name.
        """
        return cls.execute_soa_method(
            method_name='getShortcuts',
            library='Core',
            service_date='2010_04',
            service_name='Session',
            params={'shortcutInputs': shortcutInputs},
            response_cls=GetShortcutsResponse,
        )

    @classmethod
    def getPreferences2(cls, preferenceNames: List[ScopedPreferenceNames]) -> MultiPreferenceResponse2:
        """
        This operation takes an input structure which contains a scope value and vector of preference names. The return
        type of this operation is the MultiPreferencesResponse2 structure whose elements are the ServiceData and the
        vector of ReturnedPreferences2 structure.
        """
        return cls.execute_soa_method(
            method_name='getPreferences2',
            library='Core',
            service_date='2010_04',
            service_name='Session',
            params={'preferenceNames': preferenceNames},
            response_cls=MultiPreferenceResponse2,
        )


class DataManagementService(TcService):

    @classmethod
    def isPropertyLocalizable(cls, inputInfo: List[LocalizableStatusInput]) -> LocalizableStatusResponse:
        """
        The operation is used to determine if string-type property is localizable or not and can retrieve the
        localizable status for ONE or MORE properties.
        
        Use cases:
        Determine whether a string property is marked as localizable property 
        
        User needs to use this service operation to determine a string property is localizable first before he can add
        the translations to the value of this property.
        """
        return cls.execute_soa_method(
            method_name='isPropertyLocalizable',
            library='Core',
            service_date='2010_04',
            service_name='DataManagement',
            params={'inputInfo': inputInfo},
            response_cls=LocalizableStatusResponse,
        )

    @classmethod
    def setLocalizedProperties(cls, info: LocalizedPropertyValuesInfo) -> ServiceData:
        """
        This operation allows user to set or modify the display values for a localized property on a single object.
        This sets the property values for a single property on an object in different locales. With the display values
        capability, each localized string property could have different language translations associated with that.
        
        Please be aware of the following: 
        - This operation is only used to set the secondary (not the master) values of the localized property. User can
        still package the master value (with localization status marked as "M") in the 'LocalizedPropertyValuesInfo'
        structure, however, the operation will ignore and skip the master value during the process.
        - This operation is only used to set the localization values for one property. If you want to set the localized
        values for multiple properties, please use operation 'setLocalizedPropertyValues'().
        
        """
        return cls.execute_soa_method(
            method_name='setLocalizedProperties',
            library='Core',
            service_date='2010_04',
            service_name='DataManagement',
            params={'info': info},
            response_cls=ServiceData,
        )

    @classmethod
    def setLocalizedPropertyValues(cls, info: List[LocalizedPropertyValuesInfo]) -> ServiceData:
        """
        Sets the property values for multiple properties on a single object in different locales. With the display
        values capability, each localized string property could have different language translations associated with
        that. This operation allows user to set or modify the display values for the localized properties on a single
        object.
        
        It should be noted that this operation is only used to set the secondary (not the master) values of the
        localized properties. User can still package the master value (with localization status marked as "M") in the
        'LocalizedPropertyValuesInfo' structure, however, the operation will ignore and skip these master values during
        the process.
        """
        return cls.execute_soa_method(
            method_name='setLocalizedPropertyValues',
            library='Core',
            service_date='2010_04',
            service_name='DataManagement',
            params={'info': info},
            response_cls=ServiceData,
        )

    @classmethod
    def createDatasets(cls, input: List[DatasetInfo]) -> CreateDatasetsResponse:
        """
        This operation creates a list of Dataset objects, sets the requested attribute data, adds named references,
        fetches write tickets for files that will be uploaded as named references and creates the specified relation
        type between created Dataset and input container object. The caller needs to convert the structure members from
        the output Core::_2010_04::Datamanagement::CommitDatasetFileInfo to the input
        Core::_2006_03::Filemanagement::CommitDatasetFileInfo if the caller wants to use the 2010_04  version of
        createDatasets in combination with commitDatasetFiles.
        """
        return cls.execute_soa_method(
            method_name='createDatasets',
            library='Core',
            service_date='2010_04',
            service_name='DataManagement',
            params={'input': input},
            response_cls=CreateDatasetsResponse,
        )

    @classmethod
    def findDisplayableSubBusinessObjectsWithDisplayNames(cls, input: List[BOWithExclusionIn]) -> DisplayableSubBusinessObjectsResponse:
        """
        This operation returns sub Business Object names that are displayable to the login user in the object creation
        dialog and their display names for each Primary Business Object given as the input.  Returned Business Object
        lists have exclusions of Business Objects and their secondary Business Objects as specified in the input. This
        returns the hierarchy of displayable objects for each Business Object it returns.
        """
        return cls.execute_soa_method(
            method_name='findDisplayableSubBusinessObjectsWithDisplayNames',
            library='Core',
            service_date='2010_04',
            service_name='DataManagement',
            params={'input': input},
            response_cls=DisplayableSubBusinessObjectsResponse,
        )

    @classmethod
    def getAvailableTypesWithDisplayNames(cls, classes: List[BaseClassInput]) -> GetAvailableBusinessObjectTypesResponse:
        """
        This operation returns Business Object names and their display names for each primary Business Object given as
        the input.  Returned Business Object lists have exclusions of Business Objects and their secondary Business
        Objects as specified in the input. If any of the returned Business Objects is also a primary Business Object
        then this operation may not return its secondary Business Objects by default. In order to return its secondary
        Business Objects also, it is required to add this Business Object name to following preference
        TYPE_DISPLAY_RULES_list_types_of_subclasses.
        Please see the Preferences and Environment Variables Reference documentation for preference
        TYPE_DISPLAY_RULES_list_types_of_subclasses for more information.
         This is a lightweight way of getting all displayable Business Objects by name rather than model object.
        """
        return cls.execute_soa_method(
            method_name='getAvailableTypesWithDisplayNames',
            library='Core',
            service_date='2010_04',
            service_name='DataManagement',
            params={'classes': classes},
            response_cls=GetAvailableBusinessObjectTypesResponse,
        )

    @classmethod
    def getDatasetCreationRelatedInfo2(cls, typeName: str, parentObject: BusinessObject) -> GetDatasetCreationRelatedInfoResponse2:
        """
        This operation pre-populates Dataset creation information, default new Dataset name and Tool names, for a
        specified Dataset type.  This operation is used to get all the information associates with the specified
        Dataset prior to the creation operation. The returned default new Dataset name may be determined by the parent
        container object.
        """
        return cls.execute_soa_method(
            method_name='getDatasetCreationRelatedInfo2',
            library='Core',
            service_date='2010_04',
            service_name='DataManagement',
            params={'typeName': typeName, 'parentObject': parentObject},
            response_cls=GetDatasetCreationRelatedInfoResponse2,
        )

    @classmethod
    def getLocalizedProperties(cls, info: List[PropertyInfo]) -> LocalizedPropertyValuesList:
        """
        Typically business object property values are returned in the locale of the current session, this operation
        returns desired property values in any of the supported locales of the Teamcenter server.  String type
        properties may be localized with values for each supported locale, this operation will return the translated
        values for one or more desired locales.
        
        Use cases:
        Retrieve the localized values for localizable property
        
        When running Teamcenter in language environment other than the English, user wants to see the localized
        property value to be displayed in corresponding language in the UI.   This operation can be used to fulfill
        this requirement. By providing the desired business object, internal name of the properties, and specific
        locale name(s), this operation will return the localized property value(s) in that particular locale(s).
        """
        return cls.execute_soa_method(
            method_name='getLocalizedProperties',
            library='Core',
            service_date='2010_04',
            service_name='DataManagement',
            params={'info': info},
            response_cls=LocalizedPropertyValuesList,
        )


class LanguageInformationService(TcService):

    @classmethod
    def getAllTranslationStatuses(cls) -> TranslationStatusResponse:
        """
        Retrieves the full set of translation statuses: their enumeration, localized name and description. 
        Currently, the translation statuses in the Teamcenter system includes: "Master", "Approved", "Pending",
        "In-Review", and "Invalid"
        """
        return cls.execute_soa_method(
            method_name='getAllTranslationStatuses',
            library='Core',
            service_date='2010_04',
            service_name='LanguageInformation',
            params={},
            response_cls=TranslationStatusResponse,
        )

    @classmethod
    def getLanguagesList(cls, scenario: str) -> LanguageResponse:
        """
        Retrieves a list of languages according to different scenarios as specified in the input parameter. 
        All the returned language names are in the Java-standard format.
        """
        return cls.execute_soa_method(
            method_name='getLanguagesList',
            library='Core',
            service_date='2010_04',
            service_name='LanguageInformation',
            params={'scenario': scenario},
            response_cls=LanguageResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ActivateUserInput(TcBaseObj):
    """
    The input arguments for the activateUsers2 service operation.
    
    :var licenseLevel: The license level to be granted to the user. Valid values are:  
    - 0: author
    - 1: consumer
    - 2: occasional user
    - 3: viewer
    
    
    :var licenseBundle: The license bundle to be set for the user.  Bundle names are found in the license server or
    enter null for no bundle.  
    :var targetUser: The user to be activated and granted the license level, license bundle, and/or license server.
    :var licenseServer: The license server for the user.  This is a valid license server name configured by the
    administrator or null when using the site default license server.
    """
    licenseLevel: int = 0
    licenseBundle: str = ''
    targetUser: BusinessObject = None
    licenseServer: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ReportDefinition
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GenerateReportsCriteria(TcBaseObj):
    """
    Criteria needed to retrieve report definitions
    or generate report definition ids. At least one
    of the optional parameters must be included.
    
    :var clientId: The client identifier for routing purposes.
    :var rdTag: The Report Definition ID.
    :var datasetCtxUID: The uid for the context Dataset UID.
    :var datasetCtxObj: The Dataset context id.
    :var relationName: The relation name to be used.
    :var datasetType: The Dataset type to be used.
    :var reportName: Designates the name of the report given the report definition is not supplied.
    :var stylesheetTag: The report style sheet ID (optional). If no report style is provided, then the report will be
    displayed in xml format to the end user.
    :var stylesheetName: The name of the stylesheet if known
    :var contextObjects: A list of ID's representing context object(s) (required for item reports).
    :var contextObjectUIDs: A list of UIDs representing context objects.
    :var datasetName: The name of containing Dataset (optional). If the dataset name is provided, then it will save the
    generated report as a Dataset in Teamcenter database, otherwise will not.
    :var criteriaNames: The names in a series of Name/Value pairs used to specify additional criteria (optional).
    :var criteriaValues: The values in a series of Name/Value pairs used to specify additional criteria (optional).
    """
    clientId: str = ''
    rdTag: ReportDefinition = None
    datasetCtxUID: str = ''
    datasetCtxObj: BusinessObject = None
    relationName: str = ''
    datasetType: str = ''
    reportName: str = ''
    stylesheetTag: BusinessObject = None
    stylesheetName: str = ''
    contextObjects: List[BusinessObject] = ()
    contextObjectUIDs: List[str] = ()
    datasetName: str = ''
    criteriaNames: List[str] = ()
    criteriaValues: List[str] = ()


@dataclass
class GenerateReportsResponse(TcBaseObj):
    """
    The Response object from generateReports operation.
    
    :var transientFileReadTickets: It is for future use, may be null.
    :var transientFileTicketInfos: A list of TransientFileTicketInfo object(s) for retrieving generated report file.
    The TransientFileTicketInfo object includes the file name, ticket information.
    :var serviceData: The ServiceData which includes the failure information.
    """
    transientFileReadTickets: List[str] = ()
    transientFileTicketInfos: List[TransientFileTicketInfo] = ()
    serviceData: ServiceData = None


@dataclass
class TransientFileInfo(TcBaseObj):
    """
    The information required for file upload.
    
    :var fileName: Absolute file path to transient volume
    :var isText: true if filetype is text
    """
    fileName: str = ''
    isText: bool = False


@dataclass
class TransientFileTicketInfo(TcBaseObj):
    """
    TransientFileInfo with a ticket.
    
    :var transientFileInfo: file information structure
    :var ticket: FMS file ticket
    """
    transientFileInfo: TransientFileInfo = None
    ticket: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Awb0Element, Awb0ProductContextInfo, BusinessObject, RuntimeBusinessObject, RevisionRule, VariantRule, ItemRevision, Effectivity
from tcsoa.gen.Internal.ActiveWorkspaceBom._2015_07.OccurrenceManagement import EffectivityRange, OccurrenceFilterInput
from typing import Dict, List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class OccurrenceInfo5(TcBaseObj):
    """
    OccurrenceInfo5 is the information about one occurrence that is returned as part of the result. It contains an
    occurrence that is part of the configured structure being queried and that passes the filter criteria, as well as
    the attachments requested.
    
    :var occurrence: Instance of the Awb0Element.
    :var productContext: The Awb0ProductContextInfo for the occurrence. If NULL then configuration context is same as
    that of nearest parent occurrence having non-null value.
    """
    occurrence: Awb0Element = None
    productContext: Awb0ProductContextInfo = None


@dataclass
class PagedOccurrencesInfo6(TcBaseObj):
    """
    List of paged child occurrences and the cursor representing the point to which results have been retrieved.
    
    :var childOccurrences: The list of child occurrences, also represent the page of occurrences.
    :var focusOccurrence: The focused occurrence is the occurrence which needs to be searched. The focusedOccurrence is
    the Awb0Element which user wants to find with-in given product.
    :var productContext: The Awb0ProductContextInfo to be associated with the parent that was requested for expand.
    There are scenarios where the given parent may be associated with a configuration context that is different from
    the root to configure it sub level contents. If NULL then configuration context is same as that of nearest parent
    occurrence having non-null value.
    :var cursor: Cursor that represents point to which results have been retrieved.
    :var startReached: If true, the childOccurrences is considered as the first page of occurrences. This value is used
    to identify the start location of occurrence page for two way scrolling functionality.
    :var endReached: If true, the childOccurrences is considered as the last page of occurrences. This value is used to
    identify the end location of occurrence page for two way scrolling functionality.
    """
    childOccurrences: List[OccurrenceInfo5] = ()
    focusOccurrence: Awb0Element = None
    productContext: Awb0ProductContextInfo = None
    cursor: ChildOccsLoadNxtCursor5 = None
    startReached: bool = False
    endReached: bool = False


@dataclass
class ChildOccsLoadNxtCursor5(TcBaseObj):
    """
    The cusor has the information about loading the next set of child occurrences for given parents. It also has
    Awb0ProductContextInfo and input context which captures the product and configuration information.
    
    :var pageSize: This indicate the number of occurrences needs to be returned.
    :var productContext: The product context associated with the occurrence.
    :var firstLevelOnly: If true, retrieve first level occurrences only.Otherwise occurrences from multiple level to be
    retreived.
    :var parentOccurrences: Parent occurrences for which children were desired.
    :var inputCtxt: Common input context including filter, configuration, attached data criteria, page size and
    structure context object. If structure context object info is given then configuration information is not required.
    :var startIndex: Cursor start position for the result occurrences returned so far.
    :var endIndex: Cursor end position for the result occurrences returned so far.
    :var cursorData: Cursor data
    :var startOccUid: UID of the first object of the result occurrences. This is used for flat mode.
    :var endOccUid: UID of the last object of the result occurrences. This is used for flat mode.
    """
    pageSize: int = 0
    productContext: Awb0ProductContextInfo = None
    firstLevelOnly: bool = False
    parentOccurrences: List[BusinessObject] = ()
    inputCtxt: InputContext5 = None
    startIndex: int = 0
    endIndex: int = 0
    cursorData: List[str] = ()
    startOccUid: str = ''
    endOccUid: str = ''


@dataclass
class UserContextState(TcBaseObj):
    """
    This structure contains the current user's client state information for the opened object.
    
    :var openedObject: The opened object. It can be the ItemRevision, Cpd0CollaborativeDesign) or Awb0SavedBookmark
    object. The client state information like active tab, selected Awb0Element will be persisted for this object and
    the current user.
    :var sublocationAttributes: Map (string, list of strings) of client state attribute name and value pairs. At this
    time only one (key) attribute name is supported: "activeSublocation". Supported  values are:  (tab names like)
    "overview", "viewer". Keys and values are case in-sensitive. Content sublocation client state attributes that need
    to be persisted will be send to the client using this map.
    """
    openedObject: BusinessObject = None
    sublocationAttributes: SublocationAttributes = None


@dataclass
class EffectivityCreateInput(TcBaseObj):
    """
    This structure contains input to create Effectivity object for the 'element'.
    
    :var element: Awb0Element to which effectivity to be associated.
    :var effectivityInfo: Information to create Effectivity object.
    """
    element: Awb0Element = None
    effectivityInfo: EffectivityInfo = None


@dataclass
class EffectivityEditInput(TcBaseObj):
    """
    This structure contains input to update Effectivity object for the 'element'.
    
    :var element: Awb0Element to which effectivity to be associated.
    :var effectivity: The effectivity object to be updated.
    :var effectivityInfo: Information to update Effectivity object.
    """
    element: Awb0Element = None
    effectivity: Effectivity = None
    effectivityInfo: EffectivityInfo = None


@dataclass
class EffectivityInfo(TcBaseObj):
    """
    This structure contains the information required to create Effectivity.
    
    :var name: Name of the Effectivity or empty string when Effectivity should not be shared.
    :var endItemRev: Effectivity end ItemRevision.
    :var unitRangeText: Effectivity unit range, a valid range of unit numbers. Always specified in the context of the
    end Item to which the units apply. It can be a discrete, noncontinuous range.
    Valid ranges are:
      StartUnit
      StartUnit - EndUnit
      StartUnit - SO
      StartUnit - UP
            Where, StartUnit < EndUnit.
      StartUnit1 - EndUnit1, StartUnit2 - EndUnit2 (Ex. 10-12, 15-20)
            Where StartUnit2 > EndUnit1.
    All units are positive integers.
    :var dateRange: The array of effectivity date range, a valid range of dates.
    Valid ranges are:
     StartDate - EndDate
     StartDate - UP
     StartDate - SO
            Where, StartDate < EndDate and EndDate != NULL.
     StartDate1 - EndDate1, StartDate2 - EndDate2 (Ex. 01 January 2016 - 30 April 2016, 1 May 2016 - 30 July 2016)
            Where, StartDate2 > EndDate1
    :var openEndedStatus: Effectivity open ended status, 
    0 for EFFECTIVITY_closed,  
    1 for EFFECTIVITY_open_ended, 
    2 for EFFECTIVITY_stock_out.
    :var isProtected: If true, the new range is added to existing ranges, no overwrite is allowed; otherwise overwrite
    to existing ranges is allowed.
    :var isShared: If true the new Effectivity object can be shared across the different Awb0Element(s); otherwise it
    is unnamed effectivity and cannot be shared.
    """
    name: str = ''
    endItemRev: ItemRevision = None
    unitRangeText: str = ''
    dateRange: List[datetime] = ()
    openEndedStatus: int = 0
    isProtected: bool = False
    isShared: bool = False


@dataclass
class InputContext5(TcBaseObj):
    """
    'InputContext5' specifies the input information to apply to the structure from which the occurrences are to be
    retrieved. This includes configuration rules, page size of number of occurrences to fetch,  filtering and sorting
    criteria as well as criteria to fetch atachments.
    
    :var pageSize: Number of occurrences to be fetched per call.
    :var productContext: Busines Object Awb0ProductContextInfo that contains the product as well as all the
    configuration information that was passed as input to the service.
    :var configuration: A list of configuration rules to apply to the product before returning the resulting
    occurrences.
    :var filterIn: A recipe to search and filter occurrences and sort the results.
    :var occurrenceScheme: Occurrence scheme associated with product e.g Collaborative Design.
    :var changeContext: Holds the Runtime Business Object created by Server during setChangeContext2() SOA operation.
    This holds the Change Context applied and the Change Configuration Mode.
    :var requestPref: Map of preference names and value pairs (string/string). Allowed preference names are 
    includeInterfaces and includeUnconfigured. Allowed values are True/False. Keys and values are case sensitive.
    """
    pageSize: int = 0
    productContext: Awb0ProductContextInfo = None
    configuration: OccurrenceCnfgInfo5 = None
    filterIn: OccurrenceFilterInput = None
    occurrenceScheme: BusinessObject = None
    changeContext: RuntimeBusinessObject = None
    requestPref: RequestPreference5 = None


@dataclass
class OccurrenceCnfgInfo5(TcBaseObj):
    """
    OccurrenceCnfgInfo5 specifies the complete set of configuration rules including RevisionRule , Effectivity
    (provides range support) and VarianRule that need to be applied to the input product to configure it in order to
    retrieve the requested occurrence data.
    
    :var revisionRule: RevisionRule to use in order to configure the product.
    :var now: If true, specifies that the product be configure by the date that is now.
    :var effectivityDate: Specifies the date Effectivity to use in order to configure the product.
    :var unitNo: Unit number to use in order to configure the product.
    :var endItem: Specifies the endItem which governs the  unit effectivity.
    :var variantRule: VariantRule to use in order to configure the product.
    :var svrOwningProduct: This indicates the product ( ItemRevision or CollaborativeDesign ), associated with the
    saved VariantRule.
    :var effectivityRanges: List of range effectivity related data.
    """
    revisionRule: RevisionRule = None
    now: bool = False
    effectivityDate: datetime = None
    unitNo: int = 0
    endItem: BusinessObject = None
    variantRule: VariantRule = None
    svrOwningProduct: BusinessObject = None
    effectivityRanges: List[EffectivityRange] = ()


"""
The map which can have a key and value pair, used for occurrences expansion filters or criteria or options. The key and value are case sensitive. The allowed preference names are includeInterfaces. The allowed values are True/False.
"""
RequestPreference5 = Dict[str, List[str]]


"""
Map (string, list of string) of client state attribute name and value pairs. At this time only one (key) attribute name is supported: "activeSublocation". Supported  values are:  (tab names like) "overview", "viewer" . Keys and values are case in-sensitive.
"""
SublocationAttributes = Dict[str, List[str]]



from tcsoa.gen.Internal_TCXMLImportExport._2020_12.services import BriefcaseService as imp0
from tcsoa.utils import TcService


class BriefcaseService(TcService):
    exportObjectsToBriefcase = imp0.exportObjectsToBriefcase
    exportObjectsToBriefcaseAsync = imp0.exportObjectsToBriefcaseAsync
    importObjectsFromBriefcase = imp0.importObjectsFromBriefcase
    importObjectsFromBriefcaseAsync = imp0.importObjectsFromBriefcaseAsync



from __future__ import annotations

from tcsoa.gen.Internal.StructureManagement._2020_05.RevisionRuleAdministration import RevRuleInfoResponse, CreateOrUpdateRevRuleResp, CreateUpdateRevRuleInput
from tcsoa.utils import TcService


class RevisionRuleAdministrationService(TcService):

    @classmethod
    def getRevisionRuleInfo(cls, revRule: str) -> RevRuleInfoResponse:
        """
        Retreives description, rule entries and nested effectivity information for input RevisionRule.
        
        Use cases:
        User selects RevisionRule to view the details like rule entries so that user can make informed decision on
        configuring the content.
        """
        return cls.execute_soa_method(
            method_name='getRevisionRuleInfo',
            library='Internal-StructureManagement',
            service_date='2020_05',
            service_name='RevisionRuleAdministration',
            params={'revRule': revRule},
            response_cls=RevRuleInfoResponse,
        )

    @classmethod
    def createOrUpdateRevisionRule(cls, inputData: CreateUpdateRevRuleInput) -> CreateOrUpdateRevRuleResp:
        """
        Creates or updates a RevisionRule based on input information.
        
        Use cases:
        Use Case 1: Creation of Revision Rule:
        RevisionRule can be created from "Revision Rules" sub-location where all existing rules along with their
        clauses will be listed, given that user have privileges to create a RevisionRule.
        
        Use Case 2: Modification of existing Revision Rule:
        RevisionRules can be modified inside "Revision Rules" sub-location, given that user have privileges to modify a
        RevisionRule. Clauses of a RevisionRule can be re-ordered to change the precedence by moving the clause up/down
        using the corresponding controls. New clause from list of available clauses can be added or a clause can be
        removed from existing list of clauses.
        
        Use Case 3: Modification of existing Revision Rule inside ACE:
        Adhoc changes to the RevisionRule clauses can be performed to configure product structure. Addition of new
        clauses, removal of exisiting clauses or re-ordering of the clauses can be done to create a 'modified' version
        copy of the original RevisionRule which will be available to the current user only.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateRevisionRule',
            library='Internal-StructureManagement',
            service_date='2020_05',
            service_name='RevisionRuleAdministration',
            params={'inputData': inputData},
            response_cls=CreateOrUpdateRevRuleResp,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.EditContext._2015_07.DataManagement import ReferencerEditContextsResponse
from tcsoa.utils import TcService
from tcsoa.gen.BusinessObjects import WorkspaceObject


class DataManagementService(TcService):

    @classmethod
    def getReferencerEditContexts(cls, inputObjects: List[WorkspaceObject]) -> ReferencerEditContextsResponse:
        """
        This operation will return the list of context (Fnd0EditContext) objects that reference the input
        WorkspaceObjects.
        
        Use cases:
        This operation enables the following use case.
        Use Case 1: User wants to provide a list of edit contexts that can be used to configure a business object to
        enable editing in a particular context.
        - User provides the business object as the input to this operation.
        - A list of edit contexts that reference the business object are returned. Any of these edit contexts can be
        used for configuring the business object for editing in that particular context.
        
        """
        return cls.execute_soa_method(
            method_name='getReferencerEditContexts',
            library='EditContext',
            service_date='2015_07',
            service_name='DataManagement',
            params={'inputObjects': inputObjects},
            response_cls=ReferencerEditContextsResponse,
        )
